/*
 * (C) Copyright 2007, 2008
 * NEC Electronics Corporation.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Copyright 2009 Sony Corporation.
 */

#include <common.h>
#include "reg.h"
#include "spi.h"
/* 2008.12.17 fuj */
#ifdef CONFIG_SONY_ICX
#include "icx/icx.h"
#include "icx/gpio.h"
#else
#define inl(a)					(*(volatile unsigned long *)(a))
#define outl(v,a)				(*(volatile unsigned long *)(a) = (v))
#endif

#define PWC_PSCNT_ADD			0x06
#define PWC_IOSEL1_ADD			0x10
#define PWC_IOOUT1_ADD			0x14

//#define PWC_PSCNT_DATA			0x0F	/* GP2-GP5 all ON */
#define PWC_PSCNT_DATA			0x02	/* GP3 ON */
#define PWC_P7ON_DATA			0x80
#define PWC_LANRST_SET			0x80
#define PWC_LANRST_CLR			0x00


/* 2009.03.03 fuj */
#ifdef CONFIG_SONY_ICX
int icx_config_cons_index;
#endif

static void flash__init (void);
static void uart_pll_init (void);
static void timer0_init(void);
static void pwcreg_init(void);
static void ether_init (void);

int board_init (void)
{
	DECLARE_GLOBAL_DATA_PTR;

	gd->bd->bi_arch_number = MACH_TYPE_MP200;
	gd->bd->bi_boot_params = PHYS_SDRAM_1 + 0x100;

	icache_enable();

	flash__init();
	uart_pll_init();
	timer0_init();

#ifndef CONFIG_SONY_ICX
    sp0_init();
	pwcreg_init();
#endif /* !CONFIG_SONY_ICX */
	ether_init();

	return 0;
}

static void flash__init(void)
{
/* 2008.12.17 fuj */
#ifdef CONFIG_SONY_ICX
	unsigned long pinsel;

	pinsel = *(volatile unsigned long *)(CHG_PINSEL_G32);	/* GPIO_P46	*/
	*(volatile unsigned long *)(CHG_PINSEL_G32) = pinsel & 0xCFFFFFFF;

	gpio_set_value(GPIO_P46, 0x0);
	gpio_set_direction(GPIO_P46, GPIO_OUT);
#endif /* CONFIG_SONY_ICX */
}

#ifndef CONFIG_SONY_ICX
static void pwcreg_init(void)
{
    sp0_write(PWC_PSCNT_ADD,  PWC_PSCNT_DATA);		/* PWC REGGP3 -> on */
}
#endif /* !CONFIG_SONY_ICX */

static void uart_pll_init(void)
{
/* 2008.12.25 fuj */
#ifdef CONFIG_SONY_ICX
	unsigned long ul;

	/* judge port number(check P86) */
	ul=*(volatile unsigned long *)(CHG_PINSEL_G80);
	*(volatile unsigned long *)(CHG_PINSEL_G80)=(ul&0xFFFFCFFF);
	ul=*(volatile unsigned long *)(CHG_PULL_G80);
	*(volatile unsigned long *)(CHG_PULL_G80)=((ul&0xF8FFFFFF)|0x05000000);
	*(volatile unsigned long *)(GIO_E0_HH)=0x00400000;

	if (*(volatile unsigned long *)(GIO_I_HH) & 0x00400000) {
		icx_config_cons_index = 2;
		outl((ICX_BOOT_SRAM_MISC_MAGIC | ICX_BOOT_SRAM_MISC_BB), ICX_BOOT_SRAM_MISC_ADR);
	}
	else {
		icx_config_cons_index = 1;
		outl((ICX_BOOT_SRAM_MISC_MAGIC | ICX_BOOT_SRAM_MISC_SET), ICX_BOOT_SRAM_MISC_ADR);
	}

	/* setting UART port */
	ul=*(volatile unsigned long *)(CHG_PINSEL_G80);
	*(volatile unsigned long *)(CHG_PINSEL_G80)=((ul&0xFFFFC3FF)|0x00002800);
	ul=*(volatile unsigned long *)(CHG_PULL_G80);
	*(volatile unsigned long *)(CHG_PULL_G80)=((ul&0xF88FFFFF)|0x05400000);
	*(volatile unsigned long *)(ASMU_DIVU70SCLK)   = 0x00000010;	/* div 2, select PLL3 */
	*(volatile unsigned long *)(ASMU_DIVU71SCLK)   = 0x00000010;	/* div 2, select PLL3 */
	*(volatile unsigned long *)(ASMU_GCLKCTRL2ENA) = 0x000000E0;
	*(volatile unsigned long *)(ASMU_GCLKCTRL2)    = 0x000000E0;	/* U71/U70_SCLK, U70_CLK supply */
	*(volatile unsigned long *)(ASMU_GCLKCTRL2ENA) = 0x00000000;
	*(volatile unsigned long *)(ASMU_RESETREQ0ENA) = 0x18000000;
	*(volatile unsigned long *)(ASMU_RESETREQ0)    = 0x18000000;	/* U71/U70 unreset */
	*(volatile unsigned long *)(ASMU_RESETREQ0ENA) = 0x00000000;
#else
	*(volatile unsigned long *)(ASMU_DIVU70SCLK)   = 0x00000010;	/* div 2, select PLL3 */
	*(volatile unsigned long *)(ASMU_GCLKCTRL2ENA) = 0x00000060;
	*(volatile unsigned long *)(ASMU_GCLKCTRL2)    = 0x00000060;	/* U70_SCLK, U70_CLK supply */
	*(volatile unsigned long *)(ASMU_GCLKCTRL2ENA) = 0x00000000;
	*(volatile unsigned long *)(ASMU_RESETREQ0ENA) = 0x08000000;
	*(volatile unsigned long *)(ASMU_RESETREQ0)    = 0x08000000;	/* U70 unreset */
	*(volatile unsigned long *)(ASMU_RESETREQ0ENA) = 0x00000000;
#endif /* CONFIG_SONY_ICX */
}

static void ether_init(void)
{
/* 2008.12.17 fuj */
#ifndef CONFIG_SONY_ICX
    sp0_write(PWC_IOSEL1_ADD, PWC_P7ON_DATA);		/* PWC GPIO P7 -> outport */
    sp0_write(PWC_IOOUT1_ADD, PWC_LANRST_SET);		/* LAN RESET */
	udelay(1);
    sp0_write(PWC_IOOUT1_ADD, PWC_LANRST_CLR);		/* Release LAN RESET */
#endif /* !CONFIG_SONY_ICX */
}


static void timer0_init(void)
{
	*(volatile unsigned long *)(ASMU_GCLKCTRL3ENA) = 0x00004001;
	*(volatile unsigned long *)(ASMU_GCLKCTRL3)    = 0x00004001;	/* TI0_TIN_GCK supply */
	*(volatile unsigned long *)(ASMU_GCLKCTRL3ENA) = 0x00000000;
	*(volatile unsigned long *)(ASMU_RESETREQ1ENA) = 0x00000001;
	*(volatile unsigned long *)(ASMU_RESETREQ1)    = 0x00000001;	/* TI0 unreset */
	*(volatile unsigned long *)(ASMU_RESETREQ1ENA) = 0x00000000;
}


void timer0_exit(void)
{
	*(volatile unsigned long *)(ASMU_GCLKCTRL3ENA) = 0x00004001;
	*(volatile unsigned long *)(ASMU_GCLKCTRL3)    = 0x00000000;	/* TI0_TIN_GCK, TIM_TIN_GCK stop */
	*(volatile unsigned long *)(ASMU_GCLKCTRL3ENA) = 0x00000000;
}

int dram_init (void)
{
	DECLARE_GLOBAL_DATA_PTR;

	gd->bd->bi_dram[0].start = PHYS_SDRAM_1;
	gd->bd->bi_dram[0].size  = PHYS_SDRAM_1_SIZE;
	gd->bd->bi_dram[1].start = PHYS_SDRAM_2;
	gd->bd->bi_dram[1].size  = PHYS_SDRAM_2_SIZE;

	return 0;
}

/* 2008.12.17 fuj */
#ifndef CONFIG_SONY_ICX
int sp0_write(register unsigned char addr, register unsigned char data)
{
	unsigned int tx_val;

	if( !(inl(SP0_CONTROL) & SP0_CTL_TX_EMP) ) {
		sp0_init();
	}

	/* tx stop */
	outl(SP0_CTL_WRT_DIS, SP0_CONTROL);

	/* set tx data */
	tx_val = TX_DATA(addr, data);
	outl(tx_val, SP0_TX_DATA);

	/* tx start */
	outl((SP0_CTL_WRT_ENA | SP0_CTL_START), SP0_CONTROL);

	/* wait tx end */
	while( inl(SP0_CONTROL) & SP0_CTL_START ) {
	}

	/* tx stop */
	outl(SP0_CTL_WRT_DIS, SP0_CONTROL);

	return 0;
}

int sp0_init(void)
{
	/* soft reset sp0 */
	outl(SP0_CTL_RST,      SP0_CONTROL);
	udelay(1);
	outl(SP0_CTL_RST_FREE, SP0_CONTROL);

	/* initialize hw sp0-cs0 */
	outl(SP0_MODE_CS0, SP0_MODE);
	outl(SP0_POL_CS0, SP0_POL);

	return 0;
}
#endif /* !CONFIG_SONY_ICX */

