/*
 * dac_kmkr.c
 *
 * Copyright 2008,2009 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


#include <common.h>

#ifdef CONFIG_ICX

#include "../reg.h"
#include "icx_io.h"
#include "i2c.h"
#include "pwc.h"
#include "smu.h"
#include "gpio.h"
#include "icx_nvp.h"
#include "icx_beep.h"

/****************/
/*@ Definitions */
/****************/

/* GPIO */
#define	CHG_PINSEL_G80_PM0_MSK			0x0000C000
#define	CHG_PINSEL_G80_PM0				0x00004000

#define	CHG_PULL_G80_PM0_MSK			0x70000000
#define	CHG_PULL_G80_PM0				0x40000000

#define CHG_PINSEL_G00_KM_OSCEN_MSK		0x00000030
#define CHG_PINSEL_G00_KM_OSCEN			0x00000000
#define CHG_PINSEL_G00_KM_ADA_XRST_MSK	0x00000C00
#define CHG_PINSEL_G00_KM_ADA_XRST		0x00000000
#define CHG_PINSEL_G00_KM_KM_XSTBY_MSK	0x00003000
#define CHG_PINSEL_G00_KM_KM_XSTBY		0x00000000
#define CHG_PINSEL_G00_KM_HP_XMUTE_MSK	0x0000C000
#define CHG_PINSEL_G00_KM_HP_XMUTE		0x00000000

#define CHG_PULL_G00_KM_OSCEN_MSK		0x00000700
#define CHG_PULL_G00_KM_OSCEN			0x00000500
#define CHG_PULL_G00_KM_ADA_XRST_MSK	0x00700000
#define CHG_PULL_G00_KM_ADA_XRST		0x00500000
#define CHG_PULL_G00_KM_KM_XSTBY_MSK	0x07000000
#define CHG_PULL_G00_KM_KM_XSTBY		0x05000000
#define CHG_PULL_G00_KM_HP_XMUTE_MSK	0x70000000
#define CHG_PULL_G00_KM_HP_XMUTE		0x50000000

#define	ICX_BEEP_GPPUPD_HP_XMUTE2		(PWC_GPIO_GPPUPD_OPEN << PWC_GPIO_GPPUPD_SFT_IO15)
#define	ICX_BEEP_GPPUPD_HP_XMUTE2_MSK	(PWC_GPIO_GPPUPD_MSK  << PWC_GPIO_GPPUPD_SFT_IO15)

/*******************/
/*@ BEEP operation */
/*******************/

/* BEEP GAIN & FREQ */
#define BEEP_ON_REG 					0x83
#define BEEP_GAIN_REG					0xa5
#define PWM_VOL_REG_NOT_CEW 			0x47
#define PWM_VOL_REG_CEW 				0xe3

static void dac_kmkr_init(void);
static void dac_kmkr_exit(void);
static void dac_kmkr_beep_enable(void);
static void dac_kmkr_beep_disable(void);
static void dac_kmkr_beep_on(unsigned int period);
static void dac_kmkr_beep_off(void);
static void dac_kmkr_setup_platform(void);

icx_beep_ops_t dac_kmkr_ops = {
	.power_on     = dac_kmkr_init,
	.power_off    = dac_kmkr_exit,
	.beep_enable  = dac_kmkr_beep_enable,
	.beep_disable = dac_kmkr_beep_disable,
	.beep_on      = dac_kmkr_beep_on,
	.beep_off     = dac_kmkr_beep_off,
};

static i2c_config_t i2c_config = {
	.ctrl		= IIC_IICC0_WTIM0,
	.slave_adr	= 0x3C,
	.clock		= IIC_IICCL0_SMC_FAST,
	.flag		= IIC_IICF0_STCEN_EN | IIC_IICF0_IICRSV_EN,
};


/******************/
/*@ Local setting */
/******************/

#define	DAC_KMKR_REFCLK_DIV		28


/*******************/
/*@ Local routines */
/*******************/

/*-----------------------------------------------------------------------------
 *  Function    : dac_kmkr_beep_off
 *  Abstruct    : Stop to sound a beep
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
static void dac_kmkr_beep_off(void)
{
	/* stop a beep */
	i2c_write(&i2c_config, 0xF0, 0x0F);

	return;
}

/*-----------------------------------------------------------------------------
 *  Function    : dac_kmkr_beep_on
 *  Abstruct    : Sound a beep
 *  Input       : period : period to sound a beep(us)
 *				:		   (0:coutinuous)
 *  Return      : none
 *----------------------------------------------------------------------------*/
static void  dac_kmkr_beep_on(unsigned int period)
{
    
	/* sound a beep */
	i2c_write(&i2c_config, 0xF0, BEEP_ON_REG);

	if (period > 0) {
		udelay(period);
		dac_kmkr_beep_off();
	}

	return;
}

/*-----------------------------------------------------------------------------
 *  Function    : dac_kmkr_beep_enable
 *  Abstruct    : Enable the beep module(mute OFF)
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
static void dac_kmkr_beep_enable(void)
{
	/* mute off */
	gpio_set_value(BEEP_GIO_HP_XMUTE, 0x1);
	udelay(50000);
	pwc_write(PWC_GPIO_IOOUT2, BEEP_GPIO_HP_XMUTE2, BEEP_GPIO_HP_XMUTE2);

	return;
}

/*-----------------------------------------------------------------------------
 *  Function    : dac_kmkr_beep_disable
 *  Abstruct    : Disable the beep module(mute ON)
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
static void dac_kmkr_beep_disable(void)
{
	dac_kmkr_beep_off();

	/* mute on */
	pwc_write(PWC_GPIO_IOOUT2, 0x00, BEEP_GPIO_HP_XMUTE2);
	udelay(50000);
	gpio_set_value(BEEP_GIO_HP_XMUTE, 0x0);

	i2c_exit();

	return;
}

/*-----------------------------------------------------------------------------
 *  Function    : dac_kmkr_init
 *  Abstruct    : Initialize the beep module
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
static void dac_kmkr_init(void)
{
    unsigned char pwm_vol_reg = PWM_VOL_REG_NOT_CEW;
	/* setup platform */
	dac_kmkr_setup_platform();

	/* set external mute */
	gpio_set_value(BEEP_GIO_HP_XMUTE, 0x0);
	udelay(50000);
	pwc_write(PWC_GPIO_IOOUT2, 0x00, BEEP_GPIO_HP_XMUTE2);

	/* power on - digital part */
	pwc_regulator_on(PWC_REGULATOR_ANA);

	/* set reference clock */
	outl(((inl(CHG_PINSEL_REFCLKO) & ~CHG_PINSEL_REFCLKO_MSK) | CHG_PINSEL_REFCLKO_PLL2), CHG_PINSEL_REFCLKO);
	outl(DAC_KMKR_REFCLK_DIV, ASMU_DIVREFCLK);
	smu_clockgate_control(SMU_GCLKCTRL_REFCLK,  SMU_GCLKCTRL_OPEN);
	outl(DAC_KMKR_REFCLK_DIV, ASMU_DIVREFCLK);

	/* enable internal clock and unreset */
	gpio_set_value(BEEP_GIO_KM_OSCEN, 0x1);
	udelay(2000);
	gpio_set_value(BEEP_GIO_KM_XSTBY, 0x1);
	udelay(100);
	gpio_set_value(BEEP_GIO_ADA_XRST, 0x1);

	if(icx_nvp_get_ship_information()!=ICX_NVP_SYS_SHP_CEW){
        pwm_vol_reg = PWM_VOL_REG_CEW;
	}

	/* initialize registers */
	i2c_init(&i2c_config);
	i2c_write(&i2c_config, 0xC2, 0x00);
	i2c_write(&i2c_config, 0xC3, 0x01);
	i2c_write(&i2c_config, 0xF7, pwm_vol_reg);
	i2c_write(&i2c_config, 0xFE, 0x83);
	i2c_write(&i2c_config, 0xC8, 0x14);
	i2c_write(&i2c_config, 0xC9, 0x41);
	i2c_write(&i2c_config, 0xF1, BEEP_GAIN_REG);/* beep gain */
	i2c_write(&i2c_config, 0xF0, 0x0F);
	udelay(5000);

	/* power on - analog part */
	pwc_regulator_on(PWC_REGULATOR_HP);
	udelay(10000);

	/* start up D-AMP */
	i2c_write(&i2c_config, 0xF2, 0x01);
	udelay(200000);

	/* setting the mute and beep */
	i2c_write(&i2c_config, 0xF2, 0x09);
	i2c_write(&i2c_config, 0xC9, 0x01);
}

/*-----------------------------------------------------------------------------
 *  Function    : dac_kmkr_exit
 *  Abstruct    : Exit the beep module
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
static void dac_kmkr_exit(void)
{
	/* mute on */
	pwc_write(PWC_GPIO_IOOUT2, 0x00, BEEP_GPIO_HP_XMUTE2);
	udelay(50000);
	gpio_set_value(BEEP_GIO_HP_XMUTE, 0x0);
	udelay(50000);

	gpio_set_value(BEEP_GIO_ADA_XRST, 0x0);
	gpio_set_value(BEEP_GIO_KM_XSTBY, 0x0);

	pwc_regulator_off(PWC_REGULATOR_HP);
	pwc_regulator_off(PWC_REGULATOR_ANA);

	return;
}

/*-----------------------------------------------------------------------------
 *  Function    : dac_kmkr_setup_platform
 *  Abstruct    : setup platform
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
static void dac_kmkr_setup_platform(void)
{
	u_int regval;

	/****************/
	/* setting GPIO */
	/****************/

	/* set pinsel */
	regval = inl(CHG_PINSEL_G80) & ~CHG_PINSEL_G80_PM0_MSK;
	outl((regval | CHG_PINSEL_G80_PM0), CHG_PINSEL_G80);

	regval = inl(CHG_PINSEL_G00) & ~(CHG_PINSEL_G00_KM_OSCEN_MSK    |
									 CHG_PINSEL_G00_KM_ADA_XRST_MSK |
									 CHG_PINSEL_G00_KM_KM_XSTBY_MSK |
									 CHG_PINSEL_G00_KM_HP_XMUTE_MSK);
	outl((regval | (CHG_PINSEL_G00_KM_OSCEN    |
					CHG_PINSEL_G00_KM_ADA_XRST |
					CHG_PINSEL_G00_KM_KM_XSTBY |
					CHG_PINSEL_G00_KM_HP_XMUTE)), CHG_PINSEL_G00);

	pwc_write(PWC_GPIO_IOSEL2, PWC_GPIO_IOSEL_OUT, BEEP_GPIO_HP_XMUTE2);

	/* set pull */
	regval = inl(CHG_PULL_G80) & ~CHG_PULL_G80_PM0_MSK;
	outl((regval | CHG_PULL_G80_PM0), CHG_PULL_G80);

	regval = inl(CHG_PULL_G00) & ~(CHG_PULL_G00_KM_OSCEN_MSK    |
								   CHG_PULL_G00_KM_ADA_XRST_MSK |
								   CHG_PULL_G00_KM_KM_XSTBY_MSK |
								   CHG_PULL_G00_KM_HP_XMUTE_MSK);
	outl((regval | (CHG_PULL_G00_KM_OSCEN    |
					CHG_PULL_G00_KM_ADA_XRST |
					CHG_PULL_G00_KM_KM_XSTBY |
					CHG_PULL_G00_KM_HP_XMUTE)), CHG_PULL_G00);

	pwc_write(PWC_GPIO_GPPUPD4, ICX_BEEP_GPPUPD_HP_XMUTE2, ICX_BEEP_GPPUPD_HP_XMUTE2_MSK);

	/* in/out */
	gpio_set_value(BEEP_GIO_KM_OSCEN, 0x0);
	gpio_set_direction(BEEP_GIO_KM_OSCEN, GPIO_OUT);
	gpio_set_value(BEEP_GIO_ADA_XRST, 0x0);
	gpio_set_direction(BEEP_GIO_ADA_XRST, GPIO_OUT);
	gpio_set_value(BEEP_GIO_KM_XSTBY, 0x0);
	gpio_set_direction(BEEP_GIO_KM_XSTBY, GPIO_OUT);
	gpio_set_value(BEEP_GIO_HP_XMUTE, 0x0);
	gpio_set_direction(BEEP_GIO_HP_XMUTE, GPIO_OUT);
}

#endif /* CONFIG_ICX */

