/*
 * dac_l56.c
 *
 * Copyright 2008,2009 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <common.h>

#ifdef CONFIG_ICX

#include "../reg.h"
#include "icx_io.h"
#include "i2c.h"
#include "pwc.h"
#include "smu.h"
#include "gpio.h"
#include "spi.h"
#include "icx_beep.h"

/****************/
/*@ Definitions */
/****************/

/* for debug */
//#define DAC_L56_BEEP_DEBUG

/* CLOCK SELECT */
#define DAC_L56_CLK_OSC

/* GPIO */
#define	CHG_PINSEL_G80_PM0_MSK		0x0000C000
#define	CHG_PINSEL_G80_PM0			0x00004000

#define	CHG_PULL_G80_PM0_MSK		0x70000000
#define	CHG_PULL_G80_PM0			0x40000000

#define CHG_PINSEL_G00_ADA_XRST_MSK	0x00000C00
#define CHG_PINSEL_G00_ADA_XRST		0x00000000
#define CHG_PINSEL_G00_HP_XMUTE_MSK	0x0000C000
#define CHG_PINSEL_G00_HP_XMUTE		0x00000000

#define CHG_PULL_G00_ADA_XRST_MSK	0x00700000
#define CHG_PULL_G00_ADA_XRST		0x00500000
#define CHG_PULL_G00_HP_XMUTE_MSK	0x70000000
#define CHG_PULL_G00_HP_XMUTE		0x50000000

/* CHG_PINSEL_G64 : GIO_P78 to SP1_CS2 */
#define CHG_PINSEL_G64_SP1_CS2_MASK	0x30000000		// GIO_P78 Mask
#define CHG_PINSEL_G64_SP1_CS2		0x20000000		// CS2     Select

/*******************/
/*@ BEEP operation */
/*******************/
static void dac_l56_init(void);
static void dac_l56_exit(void);
static void dac_l56_beep_enable(void);
static void dac_l56_beep_disable(void);
static void dac_l56_beep_on(unsigned int period);
static void dac_l56_beep_off(void);
static void dac_l56_setup_platform(void);

icx_beep_ops_t dac_l56_ops = {
	.power_on     = dac_l56_init,
	.power_off    = dac_l56_exit,
	.beep_enable  = dac_l56_beep_enable,
	.beep_disable = dac_l56_beep_disable,
	.beep_on      = dac_l56_beep_on,
	.beep_off     = dac_l56_beep_off,
};

static spi_config_t	spi_config = {	/* asm-arm/arch-mp200/spi.h  */
	.dev = SPI_DEV_SP1,				/* device   : SPI_DEV_SP0/1				*/
	.clk = SPI_CLK_6MHZ,			/* clock								*/
	.cs  = SPI_CS_SEL_CS2,			/* cs select: SPI_CS_SEL_CS0/1/2/3		*/
	.pol = (  SPI_CK_DLY_OFF		/* polarity : CSW/CK_DLY/CK_POL/CS_POL	*/
			| SPI_CK_POL_NEG
			| SPI_CS_POL_NEG ),
	.nbw = SPI_NB_24BIT,			/* the number of bit to write			*/
	.nbr = 0,						/* the number of bit to read			*/
};

/******************/
/*@ Local setting */
/******************/
#define DAC_L56_REFCLK_OSC_OFF			(0)
#define DAC_L56_REFCLK_OSC_ON			(1)
#define DAC_L56_REFCLK_PLL2_OFF			(0xFF)
#define DAC_L56_REFCLK_PLL2_ON			(0x00)
#define	DAC_L56_REFCLK_DIV				(0x28)		/*  40 + 1 */
#define DAC_L56_REFCLK_DFLT				(0x28)		/*  40 + 1 */
#define DAC_L56_REFCLK_MLTPLY_PLL2_DFLT	(0x74)		/* 116 + 1 */
#define DAC_L56_REFCLK_MLTPLY_PLL2		(0x70)		/* 112 + 1 */

/** SPI configuration **/
#define SPI_DATA(r, d)  (u_int)((0x94<<16) | (r<<8) | d)
#define SPI_WRITE(r, d) 				\
{										\
	uint data;							\
	data = SPI_DATA((r), (d));			\
	spi_write(&spi_config, &data, 1);	\
}

/*******************/
/*@ Local routines */
/*******************/

/*-----------------------------------------------------------------------------
 *  Function    : dac_l56_beep_off
 *  Abstruct    : Stop to sound a beep
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
static void dac_l56_beep_off(void)
{
#ifdef DAC_L56_BEEP_DEBUG
return;
#endif
	/* stop a beep */
	SPI_WRITE(0x17, 0x00);

	return;
}

/*-----------------------------------------------------------------------------
 *  Function    : dac_l56_beep_on
 *  Abstruct    : Sound a beep
 *  Input       : period : period to sound a beep(us)
 *				:		   (0:coutinuous)
 *  Return      : none
 *----------------------------------------------------------------------------*/
static void  dac_l56_beep_on(unsigned int period)
{
#ifdef DAC_L56_BEEP_DEBUG
return;
#endif
	/* sound a beep */
	SPI_WRITE(0x28, 0x6c);	/* 28: Headphone-A Volume Control [ -20 dB ] */
	SPI_WRITE(0x29, 0x6c);	/* 29: Headphone-B Volume Control [ -20 dB ] */
	SPI_WRITE(0x17, 0xC0);
	if (period > 0) {
		udelay(period);
		dac_l56_beep_off();
	}
	return;
}

/*-----------------------------------------------------------------------------
 *  Function    : dac_l56_beep_enable
 *  Abstruct    : Enable the beep module(mute OFF)
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
static void dac_l56_beep_enable(void)
{
#ifdef DAC_L56_BEEP_DEBUG
return;
#endif
	/* mute off */
	gpio_set_value(BEEP_GIO_HP_XMUTE, 0x1);
	return;
}

/*-----------------------------------------------------------------------------
 *  Function    : dac_l56_beep_disable
 *  Abstruct    : Disable the beep module(mute ON)
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
static void dac_l56_beep_disable(void)
{
#ifdef DAC_L56_BEEP_DEBUG
return;
#endif
	dac_l56_beep_off();

	/* mute on */
	gpio_set_value(BEEP_GIO_HP_XMUTE, 0x0);
	return;
}

/*-----------------------------------------------------------------------------
 *  Function    : dac_l56_init
 *  Abstruct    : Initialize the beep module
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
static void dac_l56_init(void)
{
#ifdef DAC_L56_BEEP_DEBUG
return;
#endif
	/* setup platform */
	dac_l56_setup_platform();

	/* power on */
	pwc_regulator_on(PWC_REGULATOR_HP);
	udelay(10000);

	/* open clock gate */
#ifdef DAC_L56_CLK_OSC
	//icx_refclk_osc_on();
	outl(((inl(CHG_PINSEL_REFCLKO) & ~CHG_PINSEL_REFCLKO_MSK) | CHG_PINSEL_REFCLKO_OSC), CHG_PINSEL_REFCLKO);
	outl(DAC_L56_REFCLK_OSC_ON, CHG_CTRL_OSC);
	//icx_refclk_pll_off();
	outl(DAC_L56_REFCLK_MLTPLY_PLL2_DFLT, ASMU_PLL2CTRL0);
	outl(DAC_L56_REFCLK_DFLT, ASMU_DIVREFCLK);
	outl(DAC_L56_REFCLK_PLL2_OFF, ASMU_PLL2CTRL1);
#else
	/* osc off */
	outl(((inl(CHG_PINSEL_REFCLKO) & ~CHG_PINSEL_REFCLKO_MSK) | CHG_PINSEL_REFCLKO_PLL2), CHG_PINSEL_REFCLKO);
	outl(DAC_L56_REFCLK_OSC_OFF, CHG_CTRL_OSC);
	/* pll2 on */
	outl(DAC_L56_REFCLK_MLTPLY_PLL2, ASMU_PLL2CTRL0);
	outl(DAC_L56_REFCLK_DIV, ASMU_DIVREFCLK);
	outl(DAC_L56_REFCLK_PLL2_ON, ASMU_PLL2CTRL1);
#endif
	smu_clockgate_control(SMU_GCLKCTRL_REFCLK, SMU_GCLKCTRL_OPEN);

	/* 50 msec delay */
	udelay(50000);

	/* codec unreset */
	gpio_set_value(BEEP_GIO_ADA_XRST, 0x1);

	/* 10 msec delay */
	udelay(10000);

	/* spi initialize */
	spi_probe(&spi_config);

	/* regular configuration */
	SPI_WRITE(0x03, 0x3f);	/* 03: Power Control 1 : VBUF = BIAS = CHRG = ADC = ON, PDN ON */
	SPI_WRITE(0x04, 0xaf);	/* 04: Power Control 2 : HP always ON & Line always OFF */
	SPI_WRITE(0x05, 0x42);	/* 05: Clocking Control 1; Master, Predev 0,  Divided by 2 */
	SPI_WRITE(0x06, 0x08);	/* 06: Clocking Control 2: Auto-detect disable, MCLK/LRCK Ratio 128 */
	SPI_WRITE(0x07, 0x08);	/* 07: Serial Format : Left Justified */
	SPI_WRITE(0x08, 0xc5);	/* 08: Serial Format : Left Justified */
	SPI_WRITE(0x09, 0x0c);	/* 09: Misc. Control */

	SPI_WRITE(0x0B, 0x10);	/* 0b: Playback Control	*/
	SPI_WRITE(0x0C, 0xc0);	/* 0c: DSP Mute Control : ADC Mixer, PCM Mixer, Master Playback Mute */

	/* ADC Mixer Volume */
	SPI_WRITE(0x0D, 0x00);	/* 0d: ADC-A Mixer Volume Control [ 0 dB ] */
	SPI_WRITE(0x0E, 0x00);	/* 0e: ADC-B Mixer Volume Control [ 0 dB ] */

	/* PCM Mixer Volume */
	SPI_WRITE(0x0F, 0x87);	/* 0f: PCM-A Mixer Volume Control [ Mute ] */
	SPI_WRITE(0x10, 0x87);	/* 10: PCM-B Mixer Volume Control [ Mute ] */

	SPI_WRITE(0x11, 0x00);	/* 11: Analog Input Advisory Volume Control */
	SPI_WRITE(0x12, 0x00);	/* 12: Digital Input Advisory Volume Control */

	/* Master Volume */
	SPI_WRITE(0x13, 0x00);	/* 13: Master-A Volume Control    [ 0 dB ] */
	SPI_WRITE(0x14, 0x00);	/* 14: Master-B Volume Control    [ 0 dB ] */

	SPI_WRITE(0x15, 0xf0);	/* 15: Beep Frequency & On Time Control */
	SPI_WRITE(0x16, 0xf3);	/* 16: Beep Volume & Off Time Control */
	SPI_WRITE(0x17, 0x00);	/* 17: Beep & Tone Configuration Control */
	SPI_WRITE(0x18, 0x8b);	/* 18: Tone Control */
	SPI_WRITE(0x19, 0x00);	/* 19: ADC & PCM Channel Mixer Control */
	SPI_WRITE(0x1A, 0x00);	/* 1a: AIN Reference Configuration, ADC MUX Control */
	SPI_WRITE(0x1B, 0xa0);	/* 1b: HPF Control */
	SPI_WRITE(0x1C, 0x00);	/* 1c: Misc. ADC Control */
	SPI_WRITE(0x1D, 0x00);	/* 1d: Gain & Bial Control */
	SPI_WRITE(0x1E, 0x3d);	/* 1e: PGA-A MUX & Volume Control */
	SPI_WRITE(0x1F, 0x3d);	/* 1f: PGA-B MUX & Volume Control */
	SPI_WRITE(0x20, 0x00);	/* 20: ADC-A Attenuator Control */
	SPI_WRITE(0x21, 0x00);	/* 21: ADC-B Attenuator Control */
	SPI_WRITE(0x22, 0x00);	/* 22: ALC Enable & Attack Rate Control */
	SPI_WRITE(0x23, 0x3f);	/* 23: ALC Release Rate Control */
	SPI_WRITE(0x24, 0x00);	/* 24: ALC Threshold Control */
	SPI_WRITE(0x25, 0x00);	/* 25: Noise Gate Control */
	SPI_WRITE(0x26, 0x00);	/* 26: ALC and Limiter Soft Ramp, Zero Cross disables Control */
	SPI_WRITE(0x27, 0x00);	/* 27: Automute, Noise Cancel Config, Line & HP MUX */

	/* Headphne & Line Volume */
	SPI_WRITE(0x28, 0xec/*0x6c*/);	/* 28: Headphone-A Volume Control [ -20 dB ] */
	SPI_WRITE(0x29, 0xec/*0x6c*/);	/* 29: Headphone-B Volume Control [ -20 dB ] */
	SPI_WRITE(0x2A, 0xc3);	/* 2a: Line-A Volume Control      [  Mute  ] */
	SPI_WRITE(0x2B, 0xc3);	/* 2b: Line-B Volume Control      [  Mute  ] */

	SPI_WRITE(0x2C, 0x00);	/* 2c: Limiter Min/Max Thresholds Control */
	SPI_WRITE(0x2D, 0x7f);	/* 2d: Limiter Control, Release Rate */
	SPI_WRITE(0x2E, 0x00);	/* 2e: Limiter Attack Rate */

	SPI_WRITE(0x03, 0x3e);	/* Power Control 1 : VBUF = BIAS = CHRG = ADC = ON, PDA OFF*/

	/* external mute off */
	gpio_set_value(BEEP_GIO_HP_XMUTE, 0x1);
}

/*-----------------------------------------------------------------------------
 *  Function    : dac_l56_exit
 *  Abstruct    : Exit the beep module
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
static void dac_l56_exit(void)
{
#ifdef DAC_L56_BEEP_DEBUG
return;
#endif

	/* mute on */
	gpio_set_value(BEEP_GIO_HP_XMUTE, 0x0);

	/* 20 msec delay */
	udelay(20000);

	/* spi shutdown */
	spi_remove(&spi_config);

	/* reset */
	gpio_set_value(BEEP_GIO_ADA_XRST, 0x0);

	/* close clock */
	outl(DAC_L56_REFCLK_OSC_OFF, CHG_CTRL_OSC);
	outl(DAC_L56_REFCLK_PLL2_OFF, ASMU_PLL2CTRL1);
	smu_clockgate_control(SMU_GCLKCTRL_REFCLK, SMU_GCLKCTRL_CLOSE);

	/* poweroff */
	pwc_regulator_off(PWC_REGULATOR_HP);

	return;
}

/*-----------------------------------------------------------------------------
 *  Function    : dac_l56_setup_platform
 *  Abstruct    : setup platform
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
static void dac_l56_setup_platform(void)
{
	u_int regval;

	/****************/
	/* setting GPIO */
	/****************/
	/* set pinsel  HP_XMUTE(GIO_P7), ADA_XRST(GIO_P5) */
	regval = inl(CHG_PINSEL_G00) & ~(CHG_PINSEL_G00_ADA_XRST_MSK | CHG_PINSEL_G00_HP_XMUTE_MSK);
	outl((regval | (CHG_PINSEL_G00_ADA_XRST | CHG_PINSEL_G00_HP_XMUTE)), CHG_PINSEL_G00);
		/* in/out */
		gpio_set_value(BEEP_GIO_HP_XMUTE, 0x0);
		gpio_set_direction(BEEP_GIO_HP_XMUTE, GPIO_OUT);
		gpio_set_value(BEEP_GIO_ADA_XRST, 0x0);
		gpio_set_direction(BEEP_GIO_ADA_XRST, GPIO_OUT);
		/* pull up/down off */
		regval = inl(CHG_PULL_G00) & ~(CHG_PULL_G00_ADA_XRST_MSK | CHG_PULL_G00_HP_XMUTE_MSK);
		outl((regval | (CHG_PULL_G00_ADA_XRST | CHG_PULL_G00_HP_XMUTE)), CHG_PULL_G00);

	/* SPI CS2(GIO_P78) function set */
	regval = inl(CHG_PINSEL_G64) & ~CHG_PINSEL_G64_SP1_CS2_MASK;
	outl((regval | CHG_PINSEL_G64_SP1_CS2), CHG_PINSEL_G64);

	/* PM0_SI function set */
	regval = inl(CHG_PINSEL_G80) & ~CHG_PINSEL_G80_PM0_MSK;
	outl((regval | CHG_PINSEL_G80_PM0), CHG_PINSEL_G80);
		/* set pull */
		regval = inl(CHG_PULL_G80) & ~CHG_PULL_G80_PM0_MSK;
		outl((regval | CHG_PULL_G80_PM0), CHG_PULL_G80);
}
#endif /* CONFIG_ICX */
