/*
 * gpio.c
 *
 * Copyright 2008,2009 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <common.h>

#ifdef CONFIG_ICX

#include <asm/errno.h>

#include "../reg.h"
#include "icx_io.h"
#include "gpio.h"


/********************/
/*@ Global routines */
/********************/

/*-----------------------------------------------------------------------------
 *  Function    : gpio_set_direction
 *  Abstruct    : Set a direction of GPIO
 *  Input       : gpio		: GPIO No.
 *                dir		: input or output
 *  Return      : 0(success) / -EEINVAL
 *----------------------------------------------------------------------------*/
int gpio_set_direction(u_int gpio, int dir)
{
	u_int reg;

	if (gpio < 32)
		reg = dir ? GIO_E0_L   : GIO_E1_L;
	else if (gpio < 64)
		reg = dir ? GIO_E0_H   : GIO_E1_H;
	else if (gpio < 96)
		reg = dir ? GIO_E0_HH  : GIO_E1_HH;
	else if (gpio < 128)
		reg = dir ? GIO_E0_HHH : GIO_E1_HHH;
	else
		return -EINVAL;

	outl((1U << (gpio & 0x0000001F)), reg);

	return 0;
}

/*-----------------------------------------------------------------------------
 *  Function    : gpio_set_value
 *  Abstruct    : Set a value to GPIO
 *  Input       : gpio		: GPIO No.
 *                val		: Value
 *  Return      : 0(success) / -EINVAL
 *----------------------------------------------------------------------------*/
int gpio_set_value(u_int gpio, int val)
{
	u_int goe, gos;

	goe = 0x10000U << (gpio & 0x0000000F);
	gos = val ? (0x1U << (gpio & 0x0000000F)) : 0;

	if (gpio < 16)
		outl((goe | gos), GIO_OL_L);
	else if (gpio < 32)
		outl((goe | gos), GIO_OH_L);
	else if (gpio < 48)
		outl((goe | gos), GIO_OL_H);
	else if (gpio < 64)
		outl((goe | gos), GIO_OH_H);
	else if (gpio < 80)
		outl((goe | gos), GIO_OL_HH);
	else if (gpio < 96)
		outl((goe | gos), GIO_OH_HH);
	else if (gpio < 112)
		outl((goe | gos), GIO_OL_HHH);
	else if (gpio < 128)
		outl((goe | gos), GIO_OH_HHH);
	else
		return -EINVAL;

	return 0;
}


#endif	/* CONFIG_ICX */ 
