/*
 * icx_beep.c
 *
 * Copyright 2008,2009 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <common.h>

#ifdef CONFIG_ICX

#include <asm/errno.h>

#include "icx_io.h"

#include "../reg.h"
#include "pwc.h"
#include "gpio.h"
#include "icx_nvp.h"
#include "icx_beep.h"


/********************/
/*@ BEEP operations */
/********************/

static icx_beep_ops_t	*beep_ops = NULL;

extern icx_beep_ops_t dac_kmkr_ops;			/* KMKR		*/
extern icx_beep_ops_t dac_l56_ops;			/* L56		*/

/********************/
/*@ Global routines */
/********************/

/*-----------------------------------------------------------------------------
 *  Function    : icx_beep_init
 *  Abstruct    : Initialize BEEP driver
 *  Input       : none
 *  Return      : 0(success) / others(not found devices)
 *----------------------------------------------------------------------------*/
int icx_beep_init(void)
{
	u_int dac_type;

	/*******************/
	/* select a device */
	/*******************/

	dac_type = icx_nvp_get_sysinfo_dac();

	switch (dac_type) {
		case ICX_NVP_SYSINFO_DAC_KMKR :
			beep_ops = &dac_kmkr_ops;		/* KMKR		*/
			break;

		case ICX_NVP_SYSINFO_DAC_L56 :
			beep_ops = &dac_l56_ops;		/* L56		*/
			break;

		default :
			break;
	}

	/* power on the DAC */
	if (beep_ops) {
		beep_ops->power_on();
	}
	else {
		return -ENODEV;
	}
	
	return 0;
}

/*-----------------------------------------------------------------------------
 *  Function    : icx_beep_exit
 *  Abstruct    : Exit BEEP driver
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
void icx_beep_exit(void)
{
	if (beep_ops && beep_ops->power_off) {
		beep_ops->power_off();
	}
}

/*-----------------------------------------------------------------------------
 *  Function    : icx_beep_enable
 *  Abstruct    : Enable BEEP driver
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
void icx_beep_enable(void)
{
	if (beep_ops && beep_ops->beep_enable) {
		beep_ops->beep_enable();
	}
}

/*-----------------------------------------------------------------------------
 *  Function    : icx_beep_disable
 *  Abstruct    : Disable BEEP driver
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
void icx_beep_disable(void)
{
	if (beep_ops && beep_ops->beep_disable) {
		beep_ops->beep_disable();
	}
}

/*-----------------------------------------------------------------------------
 *  Function    : icx_beep_on
 *  Abstruct    : Sound a BEEP 
 *  Input       : period : period to sound a beep(us)
 *              :          (0:coutinuous)
 *  Return      : none
 *----------------------------------------------------------------------------*/
void icx_beep_on(unsigned int period)
{
	if (beep_ops && beep_ops->beep_on) {
		beep_ops->beep_on(period);
	}
}

/*-----------------------------------------------------------------------------
 *  Function    : icx_beep_off
 *  Abstruct    : Stop to sound a BEEP 
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
void icx_beep_off(void)
{
	if (beep_ops && beep_ops->beep_off) {
		beep_ops->beep_off();
	}
}

#endif	/* CONFIG_ICX */ 
