/*
 * icx_emmc.c
 *
 * Copyright 2008,2009 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <common.h>

#ifdef CONFIG_ICX

#include <asm/errno.h>

#include "icx_io.h"
#include "pwc.h"

#include "../../../../../kernel/top/include/asm-arm/arch-mp200/icx_emmc_map.h"


/*************************/
/* emmc_mini definitions */
/*************************/
#include "../emmc_mini/emmc/emmc.h"

/* boot information */
struct boot_info {
	unsigned long addr;     /* ddr address  */
	unsigned long block;    /* block number */
	unsigned long size;     /* size [byte]  */
	unsigned long erase;    /* erase flag   */
};

/* prototype */
extern void emmc_init(void);
extern int  emmc_read(struct boot_info *boot_info);


/********************/
/*@ Local variables */
/********************/

/* MBR */
static MBRecord icx_emmc_mbr;

/* initialized flag */
static int icx_emmc_initialized = 0;		/*  0 : not initialized			*/ 
											/*  1 : initialized				*/
											/* -1 : failed to initialize	*/
/********************/
/*@ Global routines */
/********************/

/*-----------------------------------------------------------------------------
 *  Function    : icx_emmc_init
 *  Abstruct    : Initialize eMMC driver(get MBR)
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
void icx_emmc_init(void)
{
	struct boot_info emmcinfo;
	int ret;

	/* dummy */
	pwc_regulator_on(PWC_REGULATOR_GP3);

	/************/
	/* read MBR */
	/************/

	/* set parameter */
	emmcinfo.addr  = (unsigned long)&icx_emmc_mbr;
	emmcinfo.block = 0;
	emmcinfo.size  = SDM_BLOCKLEN_VAL;
	emmcinfo.erase = 0;

	/* read MBR */
	ret = emmc_read(&emmcinfo);
	if (ret != 0) {
		icx_emmc_initialized = -1;
		return;
	}

	/* check the signature */
	if ((icx_emmc_mbr.sig[0] != 0x55) || (icx_emmc_mbr.sig[1] != 0xAA)) {
		icx_emmc_initialized = -1;
		return;
	}

	icx_emmc_initialized = 1;
}

/*-----------------------------------------------------------------------------
 *  Function    : icx_emmc_get_first_sector
 *  Abstruct    : Get a sector of first partition
 *  Input       : none
 *  Return      : > 0:sector number / -1:failed to read MBR
 *----------------------------------------------------------------------------*/
int icx_emmc_get_first_sector(void)
{
	if (icx_emmc_initialized == 0) {
		icx_emmc_init();
	}

	if (icx_emmc_initialized > 0) {
		return icx_emmc_mbr.partitionTable[0].firstSectorNumbers;
	}
	else {
		return -1;
	}
}

/*-----------------------------------------------------------------------------
 *  Function    : icx_emmc_exit
 *  Abstruct    : shutdown emmc
 *  Input       : none
 *  Return      :     0: ok
 *              : minus: error
 *----------------------------------------------------------------------------*/
int icx_emmc_exit(void)
{
	pwc_regulator_off(PWC_REGULATOR_GP3);
	return(0);
}


#endif	/* CONFIG_ICX */ 
