/*
 * icx_pm.h
 *
 * Copyright 2008,2009 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef __ICX_PM_H__
#define __ICX_PM_H__


/****************/
/*@ Definitions */
/****************/

/* power switch */
#define ICX_PM_OFF				0
#define ICX_PM_ON				1

/* boot factors */
#define	ICX_PM_FACTOR3_SFT		16

#define ICX_PM_FACTOR_KEY		PWC_INTC_FACTOR1_GPIO0
#define ICX_PM_FACTOR_HOLD		PWC_INTC_FACTOR1_GPIO1
#define ICX_PM_FACTOR_CRADLE	PWC_INTC_FACTOR1_GPIO7
#define	ICX_PM_FACTOR_RTC		((unsigned int)PWC_INTC_FACTOR3_FRTCINT << ICX_PM_FACTOR3_SFT)

/* battery status flag */
#define	ICX_PM_BATT_STATUS_VLOW	0x00000001
#define	ICX_PM_BATT_STATUS_NONE	0x00000002
#define	ICX_PM_BATT_STATUS_RTC	0x00000004

/* cancel LOWBATT */
#define	ICX_PM_LOWBATT_CHK_TIMEOUT	0
#define	ICX_PM_LOWBATT_CHK_CANCEL	1

/* battery flag */
#define	ICX_PM_BATT_FLAG_CLR		0
#define	ICX_PM_BATT_FLAG_SET		1


/*********************/
/*@ Type definitions */
/*********************/

/* date */
typedef struct {
	int year;

	union {
		int mdhm;

		struct {
			int month  :8;
			int day    :8;
			int hour   :8;
			int minute :8;
		} e;
	} tm;
} icx_pm_date_t;


/***************/
/*@ Prototypes */
/***************/

extern void			 icx_pm_init(void);
extern unsigned int  icx_pm_get_boot_factor(void);
extern unsigned char icx_pm_get_psw_state(void);
extern unsigned char icx_pm_get_vbat(void);
extern int			 icx_pm_get_vbat_threshold(unsigned char *low_batt, unsigned char *vlow_batt);
extern int			 icx_pm_check_battery_flag(void);
extern void			 icx_pm_set_battery_flag(int flag);
extern int			 icx_pm_check_cancel_vbat_low(void);
extern int			 icx_pm_check_rtc_afg(void);
extern int			 icx_pm_get_rtc_date(icx_pm_date_t *date_buf);
extern int			 icx_pm_check_rtc_date(icx_pm_date_t *date_buf);
extern void			 icx_pm_save_status_sram(unsigned int status);
extern void			 icx_pm_power_off(void);
extern void			 icx_pm_set_vbus_limit(int limit);
extern void			 icx_pm_set_charge_swicth(int flag);
extern void			 icx_pm_set_charge_voltage(int voltage);


#endif	/* __ICX_PM_H__ */ 
