/*
 * lcd_tmd_LTM020DL40.c
 *
 * Copyright 2008,2009 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <common.h>

#ifdef CONFIG_ICX

#include <asm/errno.h>

#include "icx_io.h"
#include "smu.h"
#include "spi.h"
#include "gpio.h"
#include "lcdc.h"
#include "pwc.h"
#include "pwm.h"

#include "icx_lcdc.h"


/*******************/
/*@ LCDC operation */
/*******************/

static int  lcd_ltm020dl40_power_on(void);
static void lcd_ltm020dl40_power_off(void);

icx_lcdc_ops_t lcd_tmd_ltm020dl40_ops = {
	.power_on		   = lcd_ltm020dl40_power_on,
	.power_off		   = lcd_ltm020dl40_power_off,

	.backlight_init	   = pwm_init,
	.backlight_exit	   = pwm_exit,
	.backlight_control = pwm_set_value,
};


/*****************/
/*@ LCDC setting */
/*****************/

/* hsync */
#define LCD_LTM020DL40_HAREA		LCDC_WIDTH_QVGA
#ifdef CONFIG_MP200_PLL3_238MHZ
#define	LCD_LTM020DL40_HPULSE		10
#define	LCD_LTM020DL40_HFRONTP		16
#define	LCD_LTM020DL40_HBACKP		10
#else
#define	LCD_LTM020DL40_HPULSE		10
#define	LCD_LTM020DL40_HFRONTP		30
#define	LCD_LTM020DL40_HBACKP		10
#endif
#define	LCD_LTM020DL40_HBLANK		(LCD_LTM020DL40_HFRONTP + LCD_LTM020DL40_HPULSE + LCD_LTM020DL40_HBACKP)
#define	LCD_LTM020DL40_HTOTAL		(LCD_LTM020DL40_HAREA + LCD_LTM020DL40_HBLANK)
#define	LCD_LTM020DL40_HEDGE1		(LCD_LTM020DL40_HFRONTP)
#define	LCD_LTM020DL40_HEDGE2		(LCD_LTM020DL40_HFRONTP + LCD_LTM020DL40_HPULSE)

/* vsync */
#define LCD_LTM020DL40_VAREA		LCDC_HEIGHT_QVGA
#define	LCD_LTM020DL40_VPULSE		2
#define	LCD_LTM020DL40_VFRONTP		2
#define	LCD_LTM020DL40_VBACKP		2
#define	LCD_LTM020DL40_VBLANK		(LCD_LTM020DL40_VFRONTP + LCD_LTM020DL40_VPULSE + LCD_LTM020DL40_VBACKP)
#define	LCD_LTM020DL40_VTOTAL		(LCD_LTM020DL40_VAREA + LCD_LTM020DL40_VBLANK)
#define	LCD_LTM020DL40_VEDGE1		(LCD_LTM020DL40_VFRONTP)
#define	LCD_LTM020DL40_VEDGE2		(LCD_LTM020DL40_VFRONTP + LCD_LTM020DL40_VPULSE)

/* LCDC config */
lcdc_config_t lcdc_tmd_ltm020dl40_config = {
	.width	  = LCDC_WIDTH_QVGA,
	.height   = LCDC_HEIGHT_QVGA,

#ifdef CONFIG_MP200_PLL3_238MHZ
	.clkdiv   = SMU_PLLSEL_PLL3 | SMU_DIV44,
#else
	.clkdiv   = SMU_PLLSEL_PLL3 | SMU_DIV40,
#endif

	.frameadr = LCDC_FRAMEADR_DEFAULT,
	.areaadr  = LCDC_AREAADR_DEFAULT,
	.pol	  = (LCDC_CLKPOL_NEG | LCDC_HPOL_POS | LCDC_VPOL_POS | LCDC_ENPOL_HI),
	.htotal   = LCD_LTM020DL40_HTOTAL, 
	.harea	  = LCD_LTM020DL40_HAREA,
	.hedge1   = LCD_LTM020DL40_HEDGE1, 
	.hedge2   = LCD_LTM020DL40_HEDGE2,
	.vtotal   = LCD_LTM020DL40_VTOTAL,
	.varea    = LCD_LTM020DL40_VAREA,
	.vedge1   = LCD_LTM020DL40_VEDGE1,
	.vedge2   = LCD_LTM020DL40_VEDGE2,

	.backlight = {
		.pol  = LCDC_BLPOL_NEG,
		.full = PWM_TOTAL,
	},
};


/****************/
/*@ SPI setting */
/****************/

/* SPI config */
static spi_config_t spi_config_w = {
	.dev = SPI_DEV_SP1,
	.clk = SPI_CLK_3MHZ,
	.cs  = SPI_CS_SEL_CS0,
	.pol = (SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG),
	.nbw = SPI_NB_24BIT,
	.nbr = 0,
};

/* SPI start byte */
#define LCD_LTM020DL40_SB_CMD	0x00700000	/* Set index register				*/
#define LCD_LTM020DL40_SB_DATA	0x00720000	/* Writes instruction or RAM data	*/

#define	LCD_LTM020DL40_IR_WAIT	0xFFFF		/* wait command(local rule)			*/

/* command type */
typedef struct lcd_ltm020dl40_command {
	unsigned short  ir;
	unsigned short  data;
} lcd_ltm020dl40_command_t;

/* SPI command */
static lcd_ltm020dl40_command_t spi_cmd_power_on[] = {
	{0x0028, 0x000E},		/* SWRST		*/
	{LCD_LTM020DL40_IR_WAIT, 10},
	{0x0016, 0x0014},		/* HPRCH		*/
	{0x0003, 0x0040},		/* PWRCTL		*/
	{0x005C, 0x0075},		/* PWRCTL2		*/
	{0x005D, 0xA6AE},		/* PWRCTL3		*/
	{0x005E, 0x0004},		/* PWRCTL4		*/
	{0x005F, 0x0122},		/* TIMINGSET1	*/
	{0x0061, 0x0111},		/* TIMINGSET3	*/
	{0x0064, 0x0009},		/* TIMINGSET6	*/
	{0x0065, 0x0716},		/* TIMINGSET7	*/
	{0x0066, 0x0304},		/* TIMINGSET8	*/
	{0x0052, 0x0000},		/* DRVOUT1		*/
	{0x0030, 0x0001},		/* GAMRED1		*/
	{0x0031, 0xFF7D},		/* GAMRED2		*/
	{0x0032, 0x8200},		/* GAMRED3		*/
	{0x0033, 0x7DFF},		/* GAMRED4		*/
	{0x0034, 0x0001},		/* GAMGREEN1	*/
	{0x0035, 0xFF7D},		/* GAMGREEN2	*/
	{0x0036, 0x8200},		/* GAMGREEN3	*/
	{0x0037, 0x7DFF},		/* GAMGREEN4	*/
	{0x0038, 0x0001},		/* GAMBLUE1		*/
	{0x0039, 0xFF7D},		/* GAMBLUE2		*/
	{0x003A, 0x8200},		/* GAMBLUE3		*/
	{0x003B, 0x7DFF},		/* GAMBLUE4		*/
	{0x0057, 0x0001},		/* PWRSEQCTL1	*/
};
#define	SPI_CMD_POWER_ON_NUM	(sizeof(spi_cmd_power_on)/sizeof(spi_cmd_power_on[0]))

static lcd_ltm020dl40_command_t spi_cmd_power_off[] = {
	{0x0057, 0x0000},		/* PWRSEQCTL1	*/
};
#define	SPI_CMD_POWER_OFF_NUM	(sizeof(spi_cmd_power_off)/sizeof(spi_cmd_power_off[0]))


/******************/
/*@ Local routine */
/******************/

/*-----------------------------------------------------------------------------
 *  Function    : lcd_ltm020dl40_send_command
 *  Abstruct    : Send a command & a data to LCD via SPI
 *  Input       : none
 *  Return      : <= 0(NG) / others(sent packet=success)
 *----------------------------------------------------------------------------*/
static int lcd_ltm020dl40_send_command(lcd_ltm020dl40_command_t *func)
{
	u_int buf;
	int   ret = 0;

	if (func->ir == LCD_LTM020DL40_IR_WAIT) {
		/* wait */
		udelay((unsigned long)func->data*1000);
		ret = func->data;
	}
	else {
		/* send a command */
		buf = LCD_LTM020DL40_SB_CMD | func->ir;
		ret = spi_write(&spi_config_w, &buf, 1);

		/* send a data */
		if (ret > 0) {
			buf = LCD_LTM020DL40_SB_DATA | func->data;
			ret = spi_write(&spi_config_w, &buf, 1);
		}
	}

	return ret;
}


/*******************/
/*@ LCDC operation */
/*******************/

/*-----------------------------------------------------------------------------
 *  Function    : lcd_ltm020dl40_power_on
 *  Abstruct    : Power on a LCD on NEC reference board
 *  Input       : config : pointer to LCDC configure[OUT]
 *  Return      : 0(success) / others(NG)
 *----------------------------------------------------------------------------*/
static int lcd_ltm020dl40_power_on(void)
{
	int i;
	int ret;

	udelay(1000);

	/* device reset */
	gpio_set_value(LCDC_GIO_LCD_XRST, 0x1);
	gpio_set_direction(LCDC_GIO_LCD_XRST, GPIO_OUT);

	/* LCDC power on(supply clock) */
	lcdc_hw_init();

	/* LCDC initialize */
	lcdc_hw_set(&lcdc_tmd_ltm020dl40_config);

	/* start clock */
	lcdc_hw_bussel(LCDC_BUSSEL_BLACK);
	lcdc_hw_start();
	udelay(10000);

	/* H/W power on */
	spi_probe(&spi_config_w);
	for (i = 0; i < SPI_CMD_POWER_ON_NUM; i++) {
		ret = lcd_ltm020dl40_send_command(&spi_cmd_power_on[i]);
		if (ret <= 0) {
			printf("LTM020DL40: spi_write error(%d)\n", ret);
			return ret;
		}
	}

	return 0;
}

/*-----------------------------------------------------------------------------
 *  Function    : lcd_ltm020dl40_power_off
 *  Abstruct    : Power off the LCD
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
static void lcd_ltm020dl40_power_off(void)
{
	int i;

	/* write black data */
	lcdc_hw_bussel(LCDC_BUSSEL_BLACK);

	udelay(40000);

	/* H/W power off */
	spi_probe(&spi_config_w);
	for (i = 0; i < SPI_CMD_POWER_OFF_NUM; i++) {
		lcd_ltm020dl40_send_command(&spi_cmd_power_off[i]);
	}

	udelay(85000);

	/* reset LCDC,IMC */
	lcdc_hw_stop();
	lcdc_hw_exit();

	/* reset the device */
	gpio_set_value(LCDC_GIO_LCD_XRST, 0x0);
}

#endif	/* CONFIG_ICX */ 
