/*
 * lcdc.c
 *
 * Copyright 2008,2009 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <common.h>

#ifdef CONFIG_ICX

#include "../reg.h"
#include "icx_io.h"
#include "smu.h"
#include "lcdc.h"


/***********************/
/*@ LCDC local setting */
/***********************/

/* format */
#define LCDC_FORMAT_RGB666      0
#define LCDC_FORMAT_RGB565      1
#define LCDC_OFORMAT_INIT		(LCDC_FORMAT_RGB666 << 4)
#define LCDC_IFORMAT_INIT		(LCDC_FORMAT_RGB666)

/* Qos */
#define LCDC_QOS_ENABLE			0x00000100
#define LCDC_QOS_DISABL			0x00000000
#define LCDC_QOS_VALUE			0x00000080
#define LCDC_QOS_INIT			(LCDC_QOS_ENABLE | LCDC_QOS_VALUE)

/* BACKCOLOR */
#define LCDC_BACKCOLOR_INIT     ((0x00 << 16) | (0x00 << 8) | (0x00 << 0))     /* black	*/

/* BUSSEL */
#define LCDC_BUSSEL_INIT        (LCDC_BUSSEL_BLACK)

/* status */
#define LCDC_MODESTATUS_MSK			0x00000700
#define LCDC_MODESTATUS_SFT			8
#define LCDC_MODESTATUS_LOCAL		(LCDC_BUSSEL_LOCAL	   << LCDC_MODESTATUS_SFT)
#define LCDC_MODESTATUS_WB			(LCDC_BUSSEL_WB_LOCAL  << LCDC_MODESTATUS_SFT)
#define LCDC_MODESTATUS_BLACK		(LCDC_BUSSEL_BLACK 	   << LCDC_MODESTATUS_SFT)
#define LCDC_MODESTATUS_BACKCOLOR	(LCDC_BUSSEL_BACKCOLOR << LCDC_MODESTATUS_SFT)

#define LCDC_STATUS_MSK			0x00000003
#define LCDC_STATUS_INACTIVE	0x00000000
#define LCDC_STATUS_ACTIVE		0x00000001

/* enable/disable */
/* LCD enable/disable */
#define LCDC_LCDOUT_ENABLE		0x1
#define LCDC_LCDOUT_DISABLE		0x0

/* interrupt factor */
#define LCDC_INTCLR_ALL			0x0000001F


/**********************/
/*@ IMC local setting */
/**********************/

/* control */
#define	IMC_START_MODE_SYNCHRO	0x00000000
#define	IMC_START_MODE_NOSYNC	0x00000001

#define	IMC_CONTROL_FORMAT_SFT	1

#define	IMC_LAYER_CONTROL_MSK	0x00000001
#define	IMC_LAYER_CONTROL_OFF	0x00000000
#define	IMC_LAYER_CONTROL_ON	0x00000001

/* refresh */
#define	IMC_REFRESH_ENABLE		0x00000001

/* format */
#define IMC_FORMAT_INIT			(LCDC_FORMAT_RGB666)
#define IMC_L1X_FORMAT_INIT		(LCDC_FORMAT_RGB565)
#define	IMC_BG_FORMAT_INIT		(IMC_BG_FORMAT_OPTIONAL)

/* alpha */
#define IMC_ALPHA_INIT			0x0000003F

/* size */
#define	IMC_VSIZE_SFT			16


/***************/
/*@ PIN select */
/***************/

/* G37 */
#define CHG_PINSEL_G32_MSK_LCD_XRST	0x00000C00
#define CHG_PINSEL_G32_LCD_XRST		0x00000000

#define CHG_PULL_G32_MSK_LCD_XRST   0x00700000
#define CHG_PULL_G32_LCD_XRST       0x00500000

/* G48 */
#define CHG_PINSEL_G48_MSK_LCD		0xFFFFFFF0
#define CHG_PINSEL_G48_LCD			0x55555550

#define	CHG_PULL_G48_MSK_LCD		0x77777700
#define	CHG_PULL_G48_LCD			0x55555500
#define	CHG_PULL_G56_MSK_LCD		0x77777777
#define	CHG_PULL_G56_LCD			0x55555555
#define	CHG_PULL_G64_MSK_LCD		0x77777777
#define	CHG_PULL_G64_LCD			0x55555555

/* G64 */
#define CHG_PINSEL_G64_MSK_LCD		0x0000FFFF
#define CHG_PINSEL_G64_LCD			0x00005555


/********************/
/*@ Gloval routines */
/********************/

/*-----------------------------------------------------------------------------
 *  Function    : lcdc_hw_bussel
 *  Abstruct    : Set BUSSEL
 *  Input       : bussel : bussel no.
 *  Return      : none
 *----------------------------------------------------------------------------*/
void lcdc_hw_bussel(u_int bussel)
{
	unsigned int mode;

	/* enable layer 1A */
	if ((bussel != LCDC_BUSSEL_BLACK) && (bussel != LCDC_BUSSEL_BACKCOLOR)) {
		if ((inl(IMC_L1A_CONTROL) & IMC_LAYER_CONTROL_MSK) == IMC_LAYER_CONTROL_OFF) {
			outl(IMC_LAYER_CONTROL_ON, IMC_L1A_CONTROL);
		}
	}

	/* update BUSSEL */
	mode = inl(LCDC_STATUS) & LCDC_MODESTATUS_MSK;
	if (mode != (bussel << LCDC_MODESTATUS_SFT)) {
		outl(bussel, LCDC_BUSSEL);
	}

	/* update IMC registers */
	outl(IMC_REFRESH_ENABLE, IMC_REFRESH);
}

/*-----------------------------------------------------------------------------
 *  Function    : lcdc_hw_start
 *  Abstruct    : Start LCDC
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
void lcdc_hw_start(void)
{
	if ((inl(LCDC_STATUS) & LCDC_STATUS_MSK) == LCDC_STATUS_INACTIVE) {
		outl(LCDC_LCDOUT_ENABLE, LCDC_LCDOUT);
	}
}

/*-----------------------------------------------------------------------------
 *  Function    : lcdc_hw_stop
 *  Abstruct    : Stop LCDC
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
void lcdc_hw_stop(void)
{
	if ((inl(LCDC_STATUS) & LCDC_STATUS_MSK) == LCDC_STATUS_ACTIVE) {
		outl(LCDC_LCDOUT_DISABLE, LCDC_LCDOUT);
	}
}

/*-----------------------------------------------------------------------------
 *  Function    : lcdc_hw_set
 *  Abstruct    : Setting LCDC
 *  Input       : lcdc_config : LCDC configure
 *  Return      : none
 *----------------------------------------------------------------------------*/
void lcdc_hw_set(lcdc_config_t *config)
{
	/****************/
	/* LCDC setting */
	/****************/

	/* set divisor */
	outl(config->clkdiv, ASMU_DIVLCDLCLK);

	/* LCDC control registers initialize */
	outl((LCDC_OFORMAT_INIT | (config->pol & 0xF)), LCDC_CONTROL);
	outl(LCDC_QOS_INIT,	  	  LCDC_QOS);
	outl(LCDC_IFORMAT_INIT,	  LCDC_IFORMAT);
	outl(LCDC_BACKCOLOR_INIT, LCDC_BACKCOLOR);
	outl(LCDC_BUSSEL_INIT,    LCDC_BUSSEL);

	/* LCDC size registers initialize */
	outl(config->htotal,      LCDC_HTOTAL);
	outl(config->harea,       LCDC_HAREA);
	outl(config->hedge1,      LCDC_HEDGE1);
	outl(config->hedge2,      LCDC_HEDGE2);
	outl(config->vtotal,      LCDC_VTOTAL);
	outl(config->varea,       LCDC_VAREA);
	outl(config->vedge1,      LCDC_VEDGE1);
	outl(config->vedge2,      LCDC_VEDGE2);

	/* clear interrupt facotr */
	outl(LCDC_INTCLR_ALL,   LCDC_INTFFCLR);
	
	/* disable interrupt */
	outl(LCDC_INTCLR_ALL,   LCDC_INTENCLR);

	/***************/
	/* IMC setting */
	/***************/

	/* control */
	outl((IMC_START_MODE_SYNCHRO | (IMC_FORMAT_INIT << IMC_CONTROL_FORMAT_SFT)), IMC_CONTROL);

	/* WB */
	outl(IMC_FORMAT_INIT,  IMC_WB_FORMAT);
	outl(config->areaadr, IMC_WB_AREAADR);
	outl(((config->width * OUT_BITS_PER_PIXEL) / 8), IMC_WB_HOFFSET);
	outl(((config->height << IMC_VSIZE_SFT) | config->width), IMC_WB_SIZE);

	/* L1 */
	outl(IMC_L1X_FORMAT_INIT, IMC_L1X_FORMAT);
	outl(IMC_ALPHA_INIT,  IMC_L1A_ALPHA);
	outl(((config->width * IN_BITS_PER_PIXEL) / 8), IMC_L1X_OFFSET);
	outl(config->frameadr, IMC_L1A_FRAMEADR);
	outl(((config->height << IMC_VSIZE_SFT) | config->width), IMC_L1A_SIZE);

	/* BG */
	outl(IMC_BG_FORMAT_INIT, IMC_BG_FORMAT);
	outl(((config->width * OUT_BITS_PER_PIXEL) / 8), IMC_BG_OFFSET);
}

/*-----------------------------------------------------------------------------
 *  Function    : lcdc_hw_init
 *  Abstruct    : H/W initialize (supply clock)
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
void lcdc_hw_init(void)
{
	/****************/
	/* clock supply */
	/****************/
	smu_clockgate_control(SMU_GCLKCTRL_LCD_CCLK, SMU_GCLKCTRL_OPEN);
	smu_clockgate_control(SMU_GCLKCTRL_LCD_LCLK, SMU_GCLKCTRL_OPEN);
	smu_clockgate_control(SMU_GCLKCTRL_LCD_PCLK, SMU_GCLKCTRL_OPEN);
	smu_clockgate_control(SMU_GCLKCTRL_LCD_CLK,  SMU_GCLKCTRL_OPEN);

	/*****************************/
	/* clock auto control -> OFF */
	/*****************************/
	smu_clkctrl_off(SMU_CLKCTRL_LCD_CLK);
	smu_clkctrl_off(SMU_CLKCTRL_LCD_CCLK);
	smu_clkctrl_off(SMU_CLKCTRL_LCD_PCLK);

	/***********/
	/* unreset */
	/***********/
	smu_reset_control(SMU_RESETREQ_LCD, SMU_RESETREQ_UNRESET);	

	/****************************/
	/* clock auto control -> ON */
	/****************************/
	smu_clkctrl_on(SMU_CLKCTRL_LCD_CLK);
	smu_clkctrl_on(SMU_CLKCTRL_LCD_CCLK);
	smu_clkctrl_on(SMU_CLKCTRL_LCD_PCLK);

	/***************/
	/* IMC Unreset */
	/***************/
	smu_clockgate_control(SMU_GCLKCTRL_IMC_PCLK, SMU_GCLKCTRL_OPEN);
	smu_clockgate_control(SMU_GCLKCTRL_IMC_CLK,  SMU_GCLKCTRL_OPEN);
	smu_clkctrl_off(SMU_CLKCTRL_IMC);
	smu_clkctrl_off(SMU_CLKCTRL_IMC_PCLK);
	smu_reset_control(SMU_RESETREQ_IMC, SMU_RESETREQ_UNRESET);	
	smu_clkctrl_on(SMU_CLKCTRL_IMC);
	smu_clkctrl_on(SMU_CLKCTRL_IMC_PCLK);
}

/*-----------------------------------------------------------------------------
 *  Function    : lcdc_hw_exit
 *  Abstruct    : Stop LCD H/W 
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
void lcdc_hw_exit(void)
{
	/*************/
	/* IMC Reset */
	/*************/
	smu_clkctrl_off(SMU_CLKCTRL_IMC);
	smu_clkctrl_off(SMU_CLKCTRL_IMC_PCLK);
	smu_reset_control(SMU_RESETREQ_IMC, SMU_RESETREQ_RESET);	
	smu_clockgate_control(SMU_GCLKCTRL_IMC_PCLK, SMU_GCLKCTRL_CLOSE);
	smu_clockgate_control(SMU_GCLKCTRL_IMC_CLK,  SMU_GCLKCTRL_CLOSE);

	/**************/
	/* LCDC Reset */
	/**************/
	smu_clkctrl_off(SMU_CLKCTRL_LCD_CLK);
	smu_clkctrl_off(SMU_CLKCTRL_LCD_CCLK);
	smu_clkctrl_off(SMU_CLKCTRL_LCD_PCLK);
	smu_reset_control(SMU_RESETREQ_LCD, SMU_RESETREQ_RESET);	
	smu_clockgate_control(SMU_GCLKCTRL_LCD_CCLK, SMU_GCLKCTRL_CLOSE);
	smu_clockgate_control(SMU_GCLKCTRL_LCD_LCLK, SMU_GCLKCTRL_CLOSE);
	smu_clockgate_control(SMU_GCLKCTRL_LCD_PCLK, SMU_GCLKCTRL_CLOSE);
	smu_clockgate_control(SMU_GCLKCTRL_LCD_CLK,  SMU_GCLKCTRL_CLOSE);
}

/*-----------------------------------------------------------------------------
 *  Function    : lcdc_init
 *  Abstruct    : Initialize LCDC
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
void lcdc_init(void)
{
	u_int pinsel;

	/**************/
	/* set PINSEL */
	/**************/

	/* LCDC */
	pinsel = inl(CHG_PINSEL_G48) & ~CHG_PINSEL_G48_MSK_LCD;
	outl((pinsel | CHG_PINSEL_G48_LCD), CHG_PINSEL_G48);
	pinsel = inl(CHG_PINSEL_G64) & ~CHG_PINSEL_G64_MSK_LCD;
	outl((pinsel | CHG_PINSEL_G64_LCD), CHG_PINSEL_G64);

	pinsel = inl(CHG_PULL_G48) & ~CHG_PULL_G48_MSK_LCD;
	outl((pinsel | CHG_PULL_G48_LCD), CHG_PULL_G48);
	pinsel = inl(CHG_PULL_G56) & ~CHG_PULL_G56_MSK_LCD;
	outl((pinsel | CHG_PULL_G56_LCD), CHG_PULL_G56);
	pinsel = inl(CHG_PULL_G64) & ~CHG_PULL_G64_MSK_LCD;
	outl((pinsel | CHG_PULL_G64_LCD), CHG_PULL_G64);

	/* LCD XRST */
	pinsel = inl(CHG_PINSEL_G32) & ~CHG_PINSEL_G32_MSK_LCD_XRST;
	outl((pinsel | CHG_PINSEL_G32_LCD_XRST), CHG_PINSEL_G32);
	pinsel = inl(CHG_PULL_G32) & ~CHG_PULL_G32_MSK_LCD_XRST;
	outl((pinsel | CHG_PULL_G32_LCD_XRST), CHG_PULL_G32);
}


#endif	/* CONFIG_ICX */ 
