/*
 * lcdc.h
 *
 * Copyright 2008,2009 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef __LCDC_H__
#define	__LCDC_H__

/*******************/
/*@ LCDC configure */
/*******************/

/* backlight config */
typedef struct {
	u_int	pol;
	u_int	full;
} lcdc_backlight_config_t;

/* LCDC config */
typedef struct {
	/* general */
	u_int	width;		/* Panel width							*/
	u_int	height;		/* Panel height							*/

	/* clkdiv */
	u_int	clkdiv;		/* clock divisor						*/

	/* setting to registers */
	u_int	frameadr;	/* FrameBuffer address(L1A)				*/
	u_int	areaadr;	/* area address(WB)						*/
	u_int	pol;		/* polarity : CLK|HPOL|VPOL|ENPOL		*/
	u_int	htotal;		/* the number of PXCLK for horizontal	*/
	u_int	harea;		/* the number of pixel for horizontal	*/
	u_int	hedge1;		/* the point of edge1 for horizontal	*/
	u_int	hedge2;		/* the point of edge2 for horizontal	*/
	u_int	vtotal;		/* the number of HSYNC for vertical		*/
	u_int	varea;		/* the number of pixel for vertical		*/
	u_int	vedge1;		/* the point of edge1 for vertical		*/
	u_int	vedge2;		/* the point of edge2 for vertical		*/

	/* backlight */
	lcdc_backlight_config_t backlight;
} lcdc_config_t;


/* size */
#define	LCDC_WIDTH_QVGA			240
#define	LCDC_HEIGHT_QVGA		320

#define	LCDC_WIDTH_WQVGA		240
#define	LCDC_HEIGHT_WQVGA		400

#define	LCDC_WIDTH_WVGA			800
#define	LCDC_HEIGHT_WVGA		480

#define LCDC_FRAME_SIZE(w, h)	((w * h * IN_BITS_PER_PIXEL) / 8)

/* bit per pixel */
#define BITS_PER_PIXEL_RGB666	18
#define BITS_PER_PIXEL_RGB565	16
#define IN_BITS_PER_PIXEL		(BITS_PER_PIXEL_RGB565)
#define OUT_BITS_PER_PIXEL		(BITS_PER_PIXEL_RGB666)

/* frame address */
#define	LCDC_FRAMEADR_DEFAULT	(0x32D46000) /* same to FB_FRAME_BUFFER_ADDR at em1_mem.h */
#define	LCDC_AREAADR_DEFAULT	(LCDC_FRAMEADR_DEFAULT + 0x001E0000)

/* lcdc_config_t.pol */
#define	LCDC_CLKPOL_POS			(0 << 3)
#define	LCDC_CLKPOL_NEG			(1 << 3)
#define	LCDC_HPOL_POS			(0 << 2)
#define	LCDC_HPOL_NEG			(1 << 2)
#define	LCDC_VPOL_POS			(0 << 1)
#define	LCDC_VPOL_NEG			(1 << 1)
#define	LCDC_ENPOL_HI			(0 << 0)
#define	LCDC_ENPOL_LOW			(1 << 0)

/* bussel */
#define LCDC_BUSSEL_LOCAL       0x00000000
#define LCDC_BUSSEL_WB_LOCAL    0x00000002
#define LCDC_BUSSEL_BLACK       0x00000004
#define LCDC_BUSSEL_BACKCOLOR   0x00000005

/* BG format */
#define	IMC_BG_FORMAT_RGB666	0x00000000
#define	IMC_BG_FORMAT_RGB565	0x00000001
#define	IMC_BG_FORMAT_BLACK		0x00000002
#define	IMC_BG_FORMAT_OPTIONAL	0x00000003

/* backlight */
#define	LCDC_BLPOL_POS			(1)
#define	LCDC_BLPOL_NEG			(0)


/**************/
/*@ prototype */
/**************/
extern void lcdc_init(void);
extern void lcdc_hw_init(void);
extern void lcdc_hw_exit(void);
extern void lcdc_hw_set(lcdc_config_t *lcdc_config);
extern void lcdc_hw_start(void);
extern void lcdc_hw_stop(void);
extern void lcdc_hw_bussel(u_int bussel);


#endif /* __LCDC_H__ */
