/*
 * oled_sdi_S6E63D4.c
 *
 * Copyright 2008,2009 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <common.h>

#ifdef CONFIG_ICX

#include <asm/errno.h>

#include "../reg.h"
#include "icx_io.h"
#include "smu.h"
#include "spi.h"
#include "gpio.h"
#include "pwc.h"
#include "lcdc.h"

#include "icx_lcdc.h"


/*******************/
/*@ LCDC operation */
/*******************/

static int  oled_s6e63d4_power_on(void);
static void oled_s6e63d4_power_off(void);

icx_lcdc_ops_t oled_sdi_s6e63d4_ops = {
	.power_on		   = oled_s6e63d4_power_on,
	.power_off		   = oled_s6e63d4_power_off,

	.backlight_init	   = NULL,
	.backlight_exit	   = NULL,
	.backlight_control = NULL,
};


/*****************/
/*@ LCDC setting */
/*****************/

/* hsync */
#define OLED_S6E63D4_HAREA		LCDC_WIDTH_WQVGA
#ifdef CONFIG_MP200_PLL3_238MHZ
#define	OLED_S6E63D4_HPULSE		5
#define	OLED_S6E63D4_HFRONTP	5
#define	OLED_S6E63D4_HBACKP		11
#else
#define	OLED_S6E63D4_HPULSE		12
#define	OLED_S6E63D4_HFRONTP	12
#define	OLED_S6E63D4_HBACKP		20
#endif
#define	OLED_S6E63D4_HBLANK		(OLED_S6E63D4_HFRONTP + OLED_S6E63D4_HPULSE + OLED_S6E63D4_HBACKP)
#define	OLED_S6E63D4_HTOTAL		(OLED_S6E63D4_HAREA + OLED_S6E63D4_HBLANK)
#define	OLED_S6E63D4_HEDGE1		(OLED_S6E63D4_HFRONTP)
#define	OLED_S6E63D4_HEDGE2		(OLED_S6E63D4_HFRONTP + OLED_S6E63D4_HPULSE)

/* vsync */
#define OLED_S6E63D4_VAREA		LCDC_HEIGHT_WQVGA
#define	OLED_S6E63D4_VPULSE		4
#define	OLED_S6E63D4_VFRONTP	8
#define	OLED_S6E63D4_VBACKP		8
#define	OLED_S6E63D4_VBLANK		(OLED_S6E63D4_VFRONTP + OLED_S6E63D4_VPULSE + OLED_S6E63D4_VBACKP)
#define	OLED_S6E63D4_VTOTAL		(OLED_S6E63D4_VAREA + OLED_S6E63D4_VBLANK)
#define	OLED_S6E63D4_VEDGE1		(OLED_S6E63D4_VFRONTP)
#define	OLED_S6E63D4_VEDGE2		(OLED_S6E63D4_VFRONTP + OLED_S6E63D4_VPULSE)

/* LCDC config */
lcdc_config_t lcdc_sdi_s6e63d4_config = {
	.width	  = LCDC_WIDTH_WQVGA,
	.height   = LCDC_HEIGHT_WQVGA,

#ifdef CONFIG_MP200_PLL3_238MHZ
	.clkdiv   = SMU_PLLSEL_PLL3 | SMU_DIV36,
#else
	.clkdiv   = SMU_PLLSEL_PLL3 | SMU_DIV32,
#endif

	.frameadr = LCDC_FRAMEADR_DEFAULT,
	.areaadr  = LCDC_AREAADR_DEFAULT,
	.pol	  = (LCDC_CLKPOL_NEG | LCDC_HPOL_POS | LCDC_VPOL_POS | LCDC_ENPOL_LOW),
	.htotal   = OLED_S6E63D4_HTOTAL, 
	.harea	  = OLED_S6E63D4_HAREA,
	.hedge1   = OLED_S6E63D4_HEDGE1, 
	.hedge2   = OLED_S6E63D4_HEDGE2,
	.vtotal   = OLED_S6E63D4_VTOTAL,
	.varea    = OLED_S6E63D4_VAREA,
	.vedge1   = OLED_S6E63D4_VEDGE1,
	.vedge2   = OLED_S6E63D4_VEDGE2,

	.backlight = {
		.pol  = 0,
		.full = 0,
	},
};


/****************/
/*@ SPI setting */
/****************/

/* SPI config */
static spi_config_t spi_config_w = {
	.dev = SPI_DEV_SP1,
	.clk = SPI_CLK_6MHZ,
	.cs  = SPI_CS_SEL_CS0,
	.pol = (SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG),
	.nbw = SPI_NB_24BIT,
	.nbr = 0,
};

/* SPI start byte */
#define OLED_S6E63D4_SB_CMD		0x00700000	/* Set index register				*/
#define OLED_S6E63D4_SB_DATA	0x00720000	/* Writes instruction or RAM data	*/

#define OLED_S6E63D4_IR_INDEX	0xFFFE	/* send only index(local rule)			*/
#define OLED_S6E63D4_IR_WAIT	0xFFFF	/* wait command(local rule)				*/

/* command type */
typedef struct oled_s6e63d4_command {
	unsigned short  ir;
	unsigned short  data;
} oled_s6e63d4_command_t;

/* SPI command */
static oled_s6e63d4_command_t spi_cmd_power_on[] = {
	/* Panel Condition set */
	{0x0007, 0x0800},     /* CLA interval: 8HCLK      */
	{0x0008, 0x0606},     /* CLB, CLC interval: 7HCLK */
	{0x0009, 0x7411},     /* SCTE: 20HCLK, SCWE: 10HCLK, SHE, CLTE: 0.5HCLK */
	{OLED_S6E63D4_IR_WAIT,  10},
	/* Display Condition set */
	{0x0001, 0x8801},     /* VFP set                  */
	{0x0002, 0x00C0},     /* 18bit RGB interface      */
	{0x001A, 0x0004},     /* Source driver control    */
	{0x0082, 0x0000},     /* Normal gamma1 set        */
	{0x0070, 0x5420},     /* Gamma table (Lv.5)       */
	{0x0071, 0x6220},
	{0x0072, 0x6F20},
	{0x0073, 0x151D},
	{0x0074, 0x2427},
	{0x0075, 0x0B20},
	{0x0076, 0x2526},
	{0x0077, 0x2D29},
	{0x0078, 0x2828},     
	{OLED_S6E63D4_IR_INDEX, 0x0083},
	/* Analog Power Condition set */
	{0x0011, 0x000F},     /* VREG1: 4.20V */
	{0x0012, 0x0008},     /* VCI1: 2.5V   */  
	{0x0013, 0x056A},     /* VINT: -2V, VGH: 5.2V, GVL: -7.0 */
	{0x0014, 0x4202},     /* VLOUT1: 5.0V, VLOUT2: 7.5V, VLOUT3: -10V */
	/* Standby OFF */
	{0x0010, 0x0000},
	{OLED_S6E63D4_IR_WAIT, 200},
	/* EL ON*/
	{0x000B, 0x0001},
	{OLED_S6E63D4_IR_WAIT,  60},
	/* Display On */
	{0x0005, 0x0001},
	/* Gram data Write */
	{OLED_S6E63D4_IR_INDEX, 0x0022},
};
#define	SPI_CMD_POWER_ON_NUM	(sizeof(spi_cmd_power_on)/sizeof(spi_cmd_power_on[0]))

static oled_s6e63d4_command_t spi_cmd_power_off[] = {
	{0x000B, 0x0000},		/* EL OFF					*/
	{OLED_S6E63D4_IR_WAIT,  60},
	{0x0005, 0x0000},		/* Display OFF				*/
	{OLED_S6E63D4_IR_WAIT,  50},
	{0x0010, 0x0001},		/* Standby ON				*/
};
#define	SPI_CMD_POWER_OFF_NUM	(sizeof(spi_cmd_power_off)/sizeof(spi_cmd_power_off[0]))


/******************/
/*@ Local routine */
/******************/

/*-----------------------------------------------------------------------------
 *  Function    : oled_s6e63d4_send_command
 *  Abstruct    : Send a command & a data to OLED via SPI
 *  Input       : none
 *  Return      : <= 0(NG) / others(sent packet=success)
 *----------------------------------------------------------------------------*/
static int oled_s6e63d4_send_command(oled_s6e63d4_command_t *func)
{
	u_int buf;
	int   ret = 0;

	if (func->ir == OLED_S6E63D4_IR_WAIT) {
		/* wait */
		udelay((unsigned long)func->data*1000);
		ret = func->data;
	}
	else if (func->ir == OLED_S6E63D4_IR_INDEX) {
		buf = OLED_S6E63D4_SB_CMD | func->data;
		ret = spi_write(&spi_config_w, &buf, 1);
	}
	else {
		/* send a command */
		buf = OLED_S6E63D4_SB_CMD | func->ir;
		ret = spi_write(&spi_config_w, &buf, 1);

		/* send a data */
		if (ret > 0) {
			buf = OLED_S6E63D4_SB_DATA | func->data;
			ret = spi_write(&spi_config_w, &buf, 1);
		}
	}

	return ret;
}


/*******************/
/*@ LCDC operation */
/*******************/

/*-----------------------------------------------------------------------------
 *  Function    : oled_s6e63d4_power_on
 *  Abstruct    : Power on the OLED
 *  Input       : config : pointer to LCDC configure[OUT]
 *  Return      : 0(success) / others(NG)
 *----------------------------------------------------------------------------*/
static int oled_s6e63d4_power_on(void)
{
	unsigned int regval;
	int i;
	int ret;

	/* UNREG on */
	regval = inl(CHG_PINSEL_G64) & ~CHG_PINSEL_G64_MSK_BLON;
	outl((regval | CHG_PINSEL_G64_BLON), CHG_PINSEL_G64);		/* BLON pinsel	*/

	regval = inl(CHG_PULL_G72) & ~CHG_PULL_G72_MSK_BLON;
	outl((regval | CHG_PULL_G72_BLON), CHG_PULL_G72);	/* BLON pull control	*/

	gpio_set_direction(LCDC_GIO_LCD_BLON, GPIO_OUT);	/* set BLON output		*/
	gpio_set_value(LCDC_GIO_LCD_BLON, 0x1);				/* set BLON to "H"		*/

	/* RESTB active */
	gpio_set_value(LCDC_GIO_LCD_XRST, 0x0);
	gpio_set_direction(LCDC_GIO_LCD_XRST, GPIO_OUT);
	udelay(10000);
	gpio_set_value(LCDC_GIO_LCD_XRST, 0x1);

	udelay(10000);

	/* H/W power on */
	spi_probe(&spi_config_w);
	for (i = 0; i < SPI_CMD_POWER_ON_NUM; i++) {
		ret = oled_s6e63d4_send_command(&spi_cmd_power_on[i]);
		if (ret <= 0) {
			printf("S6E63D4: spi_write error(%d)\n", ret);
			return ret;
		}
	}

	/* LCDC power on(supply clock) */
	lcdc_hw_init();

	/* LCDC initialize */
	lcdc_hw_set(&lcdc_sdi_s6e63d4_config);
	udelay(10000);

	/* start clock */
	lcdc_hw_bussel(LCDC_BUSSEL_BLACK);
	lcdc_hw_start();

	return 0;
}

/*-----------------------------------------------------------------------------
 *  Function    : oled_s6e63d4_power_off
 *  Abstruct    : Power off the LCD
 *  Input       : none
 *  Return      : none
 *----------------------------------------------------------------------------*/
static void oled_s6e63d4_power_off(void)
{
	int i;

	/* write black data */
	lcdc_hw_bussel(LCDC_BUSSEL_BLACK);

	udelay(40000);

	/* H/W power off */
	spi_probe(&spi_config_w);
	for (i = 0; i < SPI_CMD_POWER_OFF_NUM; i++) {
		oled_s6e63d4_send_command(&spi_cmd_power_off[i]);
	}

	udelay(85000);

	/* reset LCDC,IMC */
	lcdc_hw_stop();
	lcdc_hw_exit();

	/* reset the device */
	gpio_set_value(LCDC_GIO_LCD_XRST, 0x0);

	gpio_set_value(LCDC_GIO_LCD_BLON, 0x0);				/* set BLON to "L"		*/
}

#endif	/* CONFIG_ICX */ 
