/*
 * smu.h
 *
 * Copyright 2008,2009 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef __SMU_H__
#define	__SMU_H__

/**********************/
/*@ Group of register */
/**********************/

/* reset */
#define	SMU_RESETREQ0_GROUP			( 0 << 16)
#define	SMU_RESETREQ1_GROUP			( 1 << 16)
#define	SMU_RESETREQ2_GROUP			( 2 << 16)

/* auto clock control */
#define	SMU_AHBCLKCTRL0_GROUP		( 3 << 16)
//#define	SMU_AHBCLKCTRL1_GROUP		( 4 << 16)
#define	SMU_APBCLKCTRL0_GROUP		( 5 << 16)
#define	SMU_APBCLKCTRL1_GROUP		( 6 << 16)
#define	SMU_CLKCTRL_GROUP			( 7 << 16)

/* gated clock control */
#define SMU_GCLKCTRL0_GROUP			( 8 << 16)
#define SMU_GCLKCTRL1_GROUP			( 9 << 16)
#define SMU_GCLKCTRL2_GROUP			(10 << 16)
#define SMU_GCLKCTRL3_GROUP			(11 << 16)

/* mask */
#define	SMU_REG_GROUP_MASK			0xFFFF0000
#define	SMU_REG_BIT_MASK			0x0000FFFF
#define SMU_REG_MAX_BIT_LEN			32


/***********************/
/*@ Group & Bit assign */
/***********************/

/* RESETREQ0 */
#define	SMU_RESETREQ_LCD			(SMU_RESETREQ0_GROUP | 13)
#define	SMU_RESETREQ_IMC			(SMU_RESETREQ0_GROUP | 17)

/* RESETREQ1 */
#define	SMU_RESETREQ_SP0			(SMU_RESETREQ1_GROUP | 22)
#define	SMU_RESETREQ_SP1			(SMU_RESETREQ1_GROUP | 23)
#define	SMU_RESETREQ_IIC2			(SMU_RESETREQ1_GROUP | 25)

/* RESETREQ2 */
#define	SMU_RESETREQ_PWM			(SMU_RESETREQ2_GROUP |  9)

/* AHBCLKCTRL0 */
#define SMU_CLKCTRL_LCD_CLK			(SMU_AHBCLKCTRL0_GROUP | 13)
#define SMU_CLKCTRL_LCD_CCLK		(SMU_AHBCLKCTRL0_GROUP | 14)
#define SMU_CLKCTRL_IMC				(SMU_AHBCLKCTRL0_GROUP | 22)

/* APBCLKCTRL0 */
#define	SMU_CLKCTRL_LCD_PCLK		(SMU_APBCLKCTRL0_GROUP |  0)
#define	SMU_CLKCTRL_IMC_PCLK		(SMU_APBCLKCTRL0_GROUP | 15)

/* APBCLKCTRL1 */
#define	SMU_CLKCTRL_SP0_PCLK		(SMU_APBCLKCTRL1_GROUP |  7)
#define	SMU_CLKCTRL_SP1_PCLK		(SMU_APBCLKCTRL1_GROUP |  8)

/* CLKCTRL */
#define	SMU_CLKCTRL_SP0_SCLK		(SMU_CLKCTRL_GROUP |  2)
#define	SMU_CLKCTRL_SP1_SCLK		(SMU_CLKCTRL_GROUP |  3)

/* GCLKCTRL0 */
#define	SMU_GCLKCTRL_IMC_CLK		(SMU_GCLKCTRL0_GROUP | 18)
#define	SMU_GCLKCTRL_IMC_PCLK		(SMU_GCLKCTRL0_GROUP | 19)

/* GCLKCTRL1 */
#define	SMU_GCLKCTRL_LCD_CLK		(SMU_GCLKCTRL1_GROUP |  9)
#define	SMU_GCLKCTRL_LCD_PCLK		(SMU_GCLKCTRL1_GROUP | 10)
#define	SMU_GCLKCTRL_LCD_LCLK		(SMU_GCLKCTRL1_GROUP | 11)
#define	SMU_GCLKCTRL_LCD_CCLK		(SMU_GCLKCTRL1_GROUP | 12)

/* GCLKCTRL2 */
#define	SMU_GCLKCTRL_IIC2_CLK		(SMU_GCLKCTRL2_GROUP |  1)
#define	SMU_GCLKCTRL_IIC2_SCLK		(SMU_GCLKCTRL2_GROUP |  2)
#define	SMU_GCLKCTRL_REFCLK			(SMU_GCLKCTRL2_GROUP | 20)

/* GCLKCTRL3 */
#define	SMU_GCLKCTRL_SP0_PCLK		(SMU_GCLKCTRL3_GROUP | 21)
#define	SMU_GCLKCTRL_SP0_SCLK		(SMU_GCLKCTRL3_GROUP | 22)
#define	SMU_GCLKCTRL_SP1_PCLK		(SMU_GCLKCTRL3_GROUP | 23)
#define	SMU_GCLKCTRL_SP1_SCLK		(SMU_GCLKCTRL3_GROUP | 24)
#define	SMU_GCLKCTRL_PWM_PWCLK1		(SMU_GCLKCTRL3_GROUP | 29)
#define	SMU_GCLKCTRL_PWM_PCLK		(SMU_GCLKCTRL3_GROUP | 27)


/*******************/
/*@ Request values */
/*******************/

/* reset req */
#define	SMU_RESETREQ_RESET			0
#define	SMU_RESETREQ_UNRESET		1

/* gated clock */
#define	SMU_GCLKCTRL_CLOSE			0
#define	SMU_GCLKCTRL_OPEN			1

/* PLL select */
#define SMU_PLLSEL_PLL2				0x00000000
#define SMU_PLLSEL_PLL3				0x00000100

/* DIV value */
#define SMU_DIV6					0x50
#define SMU_DIV8					0x70
#define SMU_DIV10					0x90
#define SMU_DIV20					0x91
#define SMU_DIV32					0x72
#define SMU_DIV36					0x82
#define SMU_DIV40					0x92
#define SMU_DIV44					0xA2
#define SMU_DIV80					0x93
#define SMU_DIV160					0x94


/***************/
/*@ Prototypes */
/***************/

extern int smu_reset_control(u_int, u_char);
extern int smu_clkctrl_on(u_int);
extern int smu_clkctrl_off(u_int);
extern int smu_clockgate_control(u_int, u_char);


#endif /* __SMU_H__ */
