/*
 * cmd_icx.c
 *
 * Copyright 2008,2009 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <common.h>
#include <command.h>

#ifdef CONFIG_ICX

#if (CONFIG_COMMANDS & CFG_CMD_ICX)

#include "../board/mp200/em1/icx/icx.h"


/**************/
/*@ Configure */
/**************/

#define	ICX_TEST_OFF		0
#define	ICX_TEST_ON			1

#define	CFG_ICX_UBOOT_WRITE	ICX_TEST_ON
#define	CFG_ICX_NOR_TEST	ICX_TEST_ON
#define	CFG_ICX_LCD_TEST	ICX_TEST_ON
#define	CFG_ICX_KEY_TEST	ICX_TEST_ON


/******************/
/*@ eMMC commands */
/******************/

/***************/
/* Normal boot */
/***************/
int do_emmcboot(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	emmc_boot_linux(ICX_BOOT_NORMAL);				/* never return */
	return 0;
}

U_BOOT_CMD(
    emmcboot,  1,  0,  do_emmcboot,
    "emmcboot- Boot Linux from eMMC\n",
    "\n"
);

/******************/
/* Emergency boot */
/******************/
int do_fwupboot(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	emmc_boot_linux(ICX_BOOT_EMERGENCY);			/* never return */
	return 0;
}

U_BOOT_CMD(
    fwupboot,  1,  0,  do_fwupboot,
    "fwupboot- Boot Linux from eMMC in emergency F/W update mode\n",
    "\n"
);


/*******************************/
/*@ PowerIC read/write command */
/*******************************/
#include "../board/mp200/em1/icx/pwc.h"

int do_pwcrw(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	unsigned int  addr = 0;
	unsigned char data = 0;
	unsigned char mask = 0xFF;
	int ret;

	if (argc < 2) {
		printf("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	if (argv[1][0] == '-') {
		if (argv[1][1] == 'r') {
			if (argc < 3) {
				printf("Usage:\n%s\n", cmdtp->usage);
				return 1;
			}
			addr = simple_strtoul(argv[2], NULL, 16);
			ret = pwc_read(addr, &data);
			printf(" PWC(%d)[%02X] = %02X\n", ret, addr, data);
		}
		else if (argv[1][1] == 'w') {
			if (argc < 4) {
				printf("Usage:\n%s\n", cmdtp->usage);
				return 1;
			}
			if (argc >= 5) {
				mask = simple_strtoul(argv[4], NULL, 16);
			}
			addr = simple_strtoul(argv[2], NULL, 16);
			data = simple_strtoul(argv[3], NULL, 16);
			ret = pwc_write(addr, data, mask);
			printf(" PWC write(%d)\n", ret);
		}
		else {
			printf("Usage:\n%s\n", cmdtp->usage);
			return 1;
		}
	}
	else {
		printf("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	return 0;
}

U_BOOT_CMD(
    pwcrw   ,  5,  1,  do_pwcrw,
    "pwcrw   - Read/Write a data from/to PowerIC\n",
	"\n-r [address(0x00-0xFF)]\n"
	"-w [address(0x00-0xFF)] [data(0x00-0xFF)] <mask(0x00-0xFF)>\n"
	"	- PowerIC read/write\n"
);

#if (CFG_ICX_UBOOT_WRITE == ICX_TEST_ON)
/*******************/
/*@ U-Boot writing */
/*******************/
#include "../../kernel/top/include/asm-arm/arch-mp200/icx_emmc_map.h"
#include "../board/mp200/em1/emmc_mini/emmc/emmc.h"

/* boot information */
struct emmc_info {
	unsigned long addr;     /* ddr address  */
	unsigned long block;    /* block number */
	unsigned long size;     /* size [byte]  */
	unsigned long erase;    /* erase flag   */
};

int do_save_uboot(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	struct emmc_info info;
	MBRecord mbr;
	u_int size, sum;
	u_int *src;
	u_int *adr;
	u_int i;
	int ret;
	char *s;

	if (argc < 2) {
		s = getenv("fileaddr");
		src = (u_int *)simple_strtoul(s, NULL, 16);
	}
	else {
		src = (u_int *)simple_strtoul(argv[1], NULL, 16);
	}

	/************/
	/* Read MBR */
	/************/

	info.addr  = (unsigned long)&mbr;
	info.block = 0;
	info.size  = SDM_BLOCKLEN_VAL;
	info.erase = 0;
	ret = emmc_read(&info);
	if (ret != 0) {
		printf("Failed to read MBR.\n");
		return 1;
	}

	/*************/
	/* Write IPL */
	/*************/

	info.addr  = (unsigned long)src;
	info.block = mbr.partitionTable[0].firstSectorNumbers;
	info.size  = IPL_AREA_SECTORS * SDM_BLOCKLEN_VAL;
	info.erase = 0;
	ret = emmc_write(&info);
	if (ret != 0) {
		printf("Failed to write IPL.\n");
		return 1;
	}

	src = (unsigned long)src + info.size;

	/****************/
	/* Write U-Boot */
	/****************/

	size = inl(src);
	size += 8;
	size = (size + (SDM_BLOCKLEN_VAL - 1)) & ~(SDM_BLOCKLEN_VAL - 1);
	if (size > (LOADER_AREA_SECTORS * SDM_BLOCKLEN_VAL)) {
		printf("Too large the file(%d).\n", size);
		return 1;
	}
	
	sum = 0;
	adr = src + 1;
	for (i = 0; i < ((size - 4) / sizeof(int)); i++) {
		sum ^= *adr++;
	}
	if (sum != 0) {
		printf("Sum check error.\n");
		return 1;
	}

	info.addr  = (unsigned long)src;
	info.block = mbr.partitionTable[0].firstSectorNumbers + IPL_AREA_SECTORS;
	info.size  = size;
	info.erase = 0;
	ret = emmc_write(&info);
	if (ret != 0) {
		printf("Failed to write U-Boot.\n");
		return 1;
	}

	return 0;
}

U_BOOT_CMD(
    wruboot ,  2,  1,  do_save_uboot,
    "wruboot - Write U-Boot to eMMC\n",
	"<DDR address>\n"
	"	- Write U-Boot to eMMC\n"
);
#endif /* CFG_ICX_UBOOT_WRITE */

#if (CFG_ICX_NOR_TEST == ICX_TEST_ON)
/*************/
/*@ NOR test */
/*************/

#define	ICX_DIAG_NOR_START_ADR		0x00000000
#define	ICX_DIAG_NOR_SIZE_LARGE		0x01000000		/* 16Mbyte		*/
#define	ICX_DIAG_NOR_SIZE_SMALL		0x00020000		/* 128Kbyte		*/

extern flash_info_t  flash_info[];

/* 1byte write/read test */
static int icx_diag_nor_byte(unsigned char data, unsigned int size)
{
	unsigned int adr;
	int i;
	int ret;

	printf("\n<Data : %02Xh>\n", data);

	/* erase all sectors */
	ret = flash_erase(&flash_info[0], 0, flash_info[0].sector_count-1);
	if (ret != ERR_OK) {
		printf(" Failed to erase(%d)\n", ret);
		return -1;
	}

	/* write the data */
	adr = ICX_DIAG_NOR_START_ADR;
	printf ("Writing  [%08X-%08X] ... %08X", adr, (adr+size), adr);
	for (i = 0; i < size; i++) {
		ret = write_buff(&flash_info[0], &data, adr, 1);
		adr++;
		if (0 == (adr % 0x00010000)) {
			printf ("\b\b\b\b\b\b\b\b%08X", adr);
		}
	}
	printf (" done\n");

	/* verify */
	adr = ICX_DIAG_NOR_START_ADR;
	printf ("Checking [%08X-%08X] ... %08X", adr, (adr+size), adr);
	for (i = 0; i < size; i++) {
		if (*(unsigned char *)adr != data) {
			printf(" ERROR <data:%02Xh> [Address:%08Xh]\n", data, adr);
			return 1;
		}
		adr++;
		if (0 == (adr % 0x00010000)) {
			printf ("\b\b\b\b\b\b\b\b%08X", adr);
		}
	}
	printf (" done\n");

	return 0;
}

/* check the size switch and sequential data */
static int icx_diag_nor_size_switch(void)
{
	unsigned int size;
	unsigned int adr;
	unsigned int small_flag;
	unsigned int i, j;
	int ret;

	/* erase all sectors */
	ret = flash_erase(&flash_info[0], 0, flash_info[0].sector_count-1);
	if (ret != ERR_OK) {
		printf(" Failed to erase(%d)\n", ret);
		return -1;
	}

	/* write the data */
	size = ICX_DIAG_NOR_SIZE_LARGE / sizeof(int);
	adr  = ICX_DIAG_NOR_START_ADR;
	printf ("Writing  [%08X-%08X] ... %08X", adr, (adr+(size*sizeof(int))), adr);
	for (i = 0; i < size; i++) {
		ret = write_buff(&flash_info[0], (unsigned char *)&i, adr, sizeof(int));
		adr += sizeof(int);
		if (0 == (adr % 0x00010000)) {
			printf ("\b\b\b\b\b\b\b\b%08X", adr);
		}
	}
	printf (" done\n");

	/* verify */
	size = ICX_DIAG_NOR_SIZE_SMALL / sizeof(int);
	adr  = ICX_DIAG_NOR_START_ADR;

	if (*(unsigned int *)ICX_DIAG_NOR_START_ADR == *(unsigned int *)ICX_DIAG_NOR_SIZE_SMALL) {
		printf(" NOR size: SMALL\n");
		small_flag = 0;
	}
	else {
		printf(" NOR size: LARGE\n");
		small_flag = size;
	}

	printf ("Checking [%08X-%08X] ... %08X", adr, ICX_DIAG_NOR_SIZE_LARGE, adr);
	for (j = 0; j < (ICX_DIAG_NOR_SIZE_LARGE/ICX_DIAG_NOR_SIZE_SMALL); j++) {
		for (i = 0; i < size; i++) {
			if (*(unsigned int *)adr != (i+(j*small_flag))) {
				printf(" ERROR <data:%08Xh> [Address:%08Xh]\n", (i+(j*small_flag)), adr);
				return 1;
			}
			adr += sizeof(int);
			if (0 == (adr % 0x00010000)) {
				printf ("\b\b\b\b\b\b\b\b%08X", adr);
			}
		}
	}
	printf (" done\n");

	return 0;
}

/* check the address line */
static inline int icx_diag_nor_write_data_and_verify_all(
	unsigned int dirty_adr,
	unsigned char check_data)
{
	int ret;
	unsigned int adr;

	/* erase all sectors */
	ret = flash_erase(&flash_info[0], 0, flash_info[0].sector_count - 1);
	if (ret != ERR_OK) {
		printf(" Failed to erase(%d)\n", ret);
		return -1;
	}

	/* write the data */
	printf ("Writing  [%08X] ... %X\n", dirty_adr, check_data);
	ret = write_buff(&flash_info[0], &check_data, dirty_adr, 1);
	if (ret != ERR_OK) {
		printf(" write ERROR <data:%02Xh> [Address:%08Xh] ret=%d\n", check_data, dirty_adr, ret);
		return -1;
	}

	/* verify */
	adr = ICX_DIAG_NOR_START_ADR;
	printf ("Checking [%08X-%08X] ... %08X", adr, (adr + ICX_DIAG_NOR_SIZE_LARGE), adr);
	for (; adr < ICX_DIAG_NOR_SIZE_LARGE; adr++) {
		if (*(unsigned char *)adr != 0xFF) {
			if (!((adr == dirty_adr) && (*(unsigned char *)adr == check_data)))
			{
				printf(" verify ERROR <data:%02Xh> [Address:%08Xh]\n", *(unsigned char *)adr, adr);
				return 1;
			}
		}

		if (0 == (adr % 0x00010000))
		{
			printf("\b\b\b\b\b\b\b\b%08X", adr);
		}

		adr++;
	}

	printf("\b\b\b\b\b\b\b\b%08X", ICX_DIAG_NOR_SIZE_LARGE - 1);
	printf (" done\n");

	return ERR_OK;
}

static int icx_diag_nor_check_address_line(void)
{
	static const unsigned char check_data = 0x5A;
	unsigned int dirty_adr;
	int ret;

	/* check start address */
	dirty_adr = ICX_DIAG_NOR_START_ADR;

	ret = icx_diag_nor_write_data_and_verify_all(dirty_adr, check_data);
	if (ret != ERR_OK) {
		return -1;
	}

	/* check (fit to address line) */
	dirty_adr += 1;
	do {
		ret = icx_diag_nor_write_data_and_verify_all(dirty_adr, check_data);
		if (ret != ERR_OK) {
			return -1;
		}
		dirty_adr<<=1;
	} while (dirty_adr < ICX_DIAG_NOR_SIZE_LARGE);

	printf ("\nfinish!\n");

	return ERR_OK;
}


/* command proc */
int do_icx_diag_nor(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	unsigned char aging_data[4] = {0x00, 0x55, 0xAA, 0xFF};
	unsigned int size;
	unsigned int data;
	int cnt, i;
	int ret;

	/* check the parameters */
	if (argc < 2) {
		printf("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}
	if (argv[1][0] != '-') {
		printf("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	size = ICX_DIAG_NOR_SIZE_SMALL;
	data = 0;
	ret  = 0;
	switch (argv[1][1]) {
		case 'c' :
			ret = icx_diag_nor_check_address_line();
			break;
		case 'l' :
			size = ICX_DIAG_NOR_SIZE_LARGE;		/* fall through */
		case 's' :
			if (argc < 3) {
				printf("Usage:\n%s\n", cmdtp->usage);
				return 1;
			}
			data = simple_strtoul(argv[2], NULL, 16);
			ret = icx_diag_nor_byte(data, size);
			break;
		case 'a' :
			for (cnt = 0; ; cnt++) {
				printf("\n --- Iteration:%x ---\n", cnt);
				for (i = 0; i < 4; i++) {
					ret = icx_diag_nor_byte(aging_data[i], ICX_DIAG_NOR_SIZE_LARGE);
					if (ret) {
						return -1;
					}
				}
			}
			break;
		case 'i' :
			ret = icx_diag_nor_size_switch();
			break;
		default :
			printf("Usage:\n%s\n", cmdtp->usage);
			ret = 1;
			break;
	}

	if (ret == 0)
		printf(" -> OK\n\n");

	return ret;
}

/* command definition */
U_BOOT_CMD(
	norcheck,	3,	1,	do_icx_diag_nor,
	"norcheck- NOR test\n",
	"\n-l [1byte data (ex. 00/55/aa/ff)]\n"
	"-s [1byte data (ex. 00/55/aa/ff)]\n"
	"-i : 4byte incremental data(check the size switch)\n"
	"-a : aging\n"
	"-c : check address line\n"
	"	- NOR read/write test\n"
);

#endif /* CFG_ICX_NOR_TEST */

#if	(CFG_ICX_LCD_TEST == ICX_TEST_ON)
/*************/
/*@ LCD test */
/*************/

/* command proc */
int do_icx_diag_lcd(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int deg = 0;
	int ret;

	/* check the parameters */
	if (argc < 2) {
		printf("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	if (argv[1][0] == '-') {
		if (argv[1][1] == 'u') {
			icx_lcdc_draw_image(1);
		}
		else if (argv[1][1] == 'b') {
			deg = simple_strtoul(argv[2], NULL, 10);
			ret = icx_lcdc_backlight_control(deg, 5000);
			if (ret < 0) {
				printf("Invalid parameter\n");
			}
		}
		else {
			printf("Usage:\n%s\n", cmdtp->usage);
			return 1;
		}
	}
	else {
		printf("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	return ret;
}

/* command definition */
U_BOOT_CMD(
	lcd,	3,	1,	do_icx_diag_lcd,
	"lcd     - LCD test\n",
	"\n-u\n"
	"	- update image\n"
	"-b [0-100]\n"
	"	- LCD backlight test\n"
);
#endif /* CFG_ICX_LCD_TEST */

#if	(CFG_ICX_KEY_TEST == ICX_TEST_ON)
#include "../board/mp200/em1/icx/adc.h"
#include "../board/mp200/em1/icx/pwc.h"
/*************/
/*@ KEY test */
/*************/

/* command proc */
int do_icx_diag_key(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	unsigned char keycode = 0xFF;
	unsigned int  holdstat;
	int ret;

	/* get KEY */
	ret = adc_read(PWC_ADSEL_ADIN3, &keycode);
	if (ret == 0) {
		printf(" KEY_AD0 : %02Xh\n", keycode);

		ret = adc_read(PWC_ADSEL_ADIN4, &keycode);
		if (ret == 0) {
			printf(" KEY_AD1 : %02Xh\n", keycode);
		}
		else {
			printf(" Failed to get KEY_AD1(%d)\n", ret);
		}
	}
	else {
		printf(" Failed to get KEY_AD0(%d)\n", ret);
	}

	/* get HOLD */
	holdstat = icx_key_get_hold();
	printf(" HOLD : %08Xh\n", holdstat);

	return ret;
}

/* command definition */
U_BOOT_CMD(
	key,	1,	1,	do_icx_diag_key,
	"key     - KEY test\n",
	"\n"
	"	- print KEY code\n"
);
#endif /* CFG_ICX_KEY_TEST */

#endif /* (CONFIG_COMMANDS & CFG_CMD_ICX) */

#endif /* CONFIG_ICX */
